@@ -72,6 +72,9 @@
 #define ADDRESSLEN 47
 #endif
 
+//! Max allowed packet assembled size
+#define MAX_CAPTURE_LEN 20480
+
 //! Capture modes
 enum capture_status {
     CAPTURE_ONLINE = 0,
@@ -80,16 +83,25 @@ enum capture_status {
     CAPTURE_OFFLINE_LOADING,
 };
 
-//! Shorter declaration of capture_info structure
+enum capture_storage {
+    CAPTURE_STORAGE_NONE = 0,
+    CAPTURE_STORAGE_MEMORY,
+    CAPTURE_STORAGE_DISK
+};
+
+//! Shorter declaration of capture_config structure
+typedef struct capture_config capture_config_t;
+//; Shorter declaration of capture_info structure
 typedef struct capture_info capture_info_t;
 //! Shorter declaration of dns_cache structure
 typedef struct dns_cache dns_cache_t;
 //! Shorter declaration of capture_packet structure
 typedef struct capture_packet capture_packet_t;
+//! Shorter declaration of capture_frame structure
+typedef struct capture_frame capture_frame_t;
 
 //! Stored packet types
-enum capture_packet_type
-{
+enum capture_packet_type {
     CAPTURE_PACKET_SIP_UDP = 0,
     CAPTURE_PACKET_SIP_TCP,
     CAPTURE_PACKET_SIP_TLS,
@@ -111,57 +123,118 @@ struct dns_cache {
 };
 
 /**
- * @brief store all information related with packet capture
+ * @brief Capture common configuration
  *
- * All required information to capture and process packets
- * will be stored in this
+ * Store capture configuration and global data
  */
-struct capture_info {
+struct capture_config {
     //! Capture status
     int status;
     //! Calls capture limit. 0 for disabling
     int limit;
     //! Also capture RTP packets
     int rtp_capture;
-    //! Input file in Offline capture
-    const char *infile;
+    //! Where should we store captured packets
+    int storage;
     //! Key file for TLS decrypt
     const char *keyfile;
     //! The compiled filter expression
     struct bpf_program fp;
-    //! Netmask of our sniffing device
-    bpf_u_int32 mask;
-    //! The IP of our sniffing device
-    bpf_u_int32 net;
-    //! libpcap capture handler
-    pcap_t *handle;
     //! libpcap dump file handler
     pcap_dumper_t *pd;
-    //! libpcap link type
-    int link;
     //! Cache for DNS lookups
     dns_cache_t dnscache;
     //! Local devices pointer
     pcap_if_t *devices;
+    //! Capture sources
+    vector_t *sources;
+    //! Packets pending IP reassembly
+    vector_t *ip_reasm;
+    //! Packets pending TCP reassembly
+    vector_t *tcp_reasm;
+    //! Capture Lock. Avoid parsing and handling data at the same time
+    pthread_mutex_t lock;
+};
+
+/**
+ * @brief store all information related with packet capture
+ *
+ * Store capture required data from one packet source
+ */
+struct capture_info
+{
+    //! libpcap link type
+    int link;
+    //! libpcap link header size
+    int8_t link_hl;
+    //! libpcap capture handler
+    pcap_t *handle;
+    //! Netmask of our sniffing device
+    bpf_u_int32 mask;
+    //! The IP of our sniffing device
+    bpf_u_int32 net;
+    //! Input file in Offline capture
+    const char *infile;
     //! Capture thread for online capturing
     pthread_t capture_t;
 };
 
+/**
+ * Packet capture data.
+ *
+ * One packet can contain more than one frame after
+ * assembly. We assume than one SIP message has one packet
+ * (maybe in multiple frames) and that one packet can only contain
+ *  one SIP message.
+ *
+ */
 struct capture_packet {
+    // IP protocol
+    uint8_t ip_version;
+    // Transport protocol
+    uint8_t proto;
     // Packet type as defined in capture_packet_type
     int type;
-    //! PCAP Packet Header data
-    struct pcap_pkthdr *header;
-    //! PCAP Packet content
-    u_char *data;
-    //! PPCAP Packet content len
-    int size;
-    //! Byte number where payload starts
-    int payload_start;
+    // Packet source and destination address
+    char ip_src[ADDRESSLEN], ip_dst[ADDRESSLEN];
+    // Packet source and destination port
+    u_short sport, dport;
+    //! Packet IP id
+    uint16_t ip_id;
+    //! PCAP Packet payload when it can not be get from data
+    u_char *payload;
     //! Payload length
-    int payload_len;
+    uint32_t payload_len;
+    //! Packet frame list (capture_frame_t)
+    vector_t *frames;
 };
 
+/**
+ *  Capture frame.
+ *  One packet can contain multiple frames.
+ */
+struct capture_frame {
+    //! PCAP Frame Header data
+    struct pcap_pkthdr *header;
+    //! PCAP Frame content
+    u_char *data;
+};
+
+/**
+ * @brief Initialize capture data
+ *
+ * @param limit Numbers of calls >0
+ * @param rtp_catpure Enable rtp capture
+ */
+void
+capture_init(int limit, int rtp_capture);
+
+/**
+ * @brief Deinitialize capture data
+ */
+void
+capture_deinit();
+
 /**
  * @brief Online capture function
  *
@@ -193,10 +266,20 @@ capture_offline(const char *infile, const char *outfile);
  * methods using pcap. This will get the payload from a package and
  * add it to the SIP storage layer.
  *
- * @param handle LIBPCAP capture handler
  */
 void
-parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packet);
+parse_packet(u_char *capinfo, const struct pcap_pkthdr *header, const u_char *packet);
+
+/**
+ * @brief Check if the given packet structure is SIP/RTP/..
+ *
+ * This function will call parse functions to determine if packet has relevant data
+ *
+ * @return 0 in case this packets has SIP/RTP data
+ * @return 1 otherwise
+ */
+int
+capture_packet_parse(capture_packet_t *pkt);
 
 /**
  * @brief Create a capture thread for online mode
@@ -232,15 +315,6 @@ capture_is_online();
 int
 capture_set_bpf_filter(const char *filter);
 
-/**
- * @brief Set capture options
- *
- * @param limit Numbers of calls >0
- * @param rtp_catpure Enable rtp capture
- */
-void
-capture_set_opts(int limit, int rtp_capture);
-
 /**
  * @brief Pause/Resume capture
  *
@@ -300,11 +374,34 @@ capture_set_keyfile(const char *keyfile);
 char *
 capture_last_error();
 
+/**
+ * @brief Avoid parsing more packets
+ */
+void
+capture_lock();
+
+/**
+ * @brief Allow parsing more packets
+ */
+void
+capture_unlock();
+
 /**
  * @brief Allocate memory to store new packet data
  */
 capture_packet_t *
-capture_packet_create(const struct pcap_pkthdr *header, const u_char *packet, int size, int payload_len);
+capture_packet_create(uint8_t ip_ver, uint8_t proto, const char *ip_src, const char *ip_dst, uint32_t id);
+
+/**
+ * @brief Set Transport layer information
+ */
+capture_packet_t *
+capture_packet_set_transport_data(capture_packet_t *pkt, u_short sport, u_short dport, int type);
+/**
+ * @brief Add a new frame to the given packet
+ */
+capture_frame_t *
+capture_packet_add_frame(capture_packet_t *pkt, const struct pcap_pkthdr *header, const u_char *packet);
 
 /**
  * @brief Deallocate a packet structure memory
@@ -318,12 +415,44 @@ capture_packet_destroy(capture_packet_t *packet);
 void
 capture_packet_destroyer(void *packet);
 
+/**
+ * @brief Free packet frames data.
+ *
+ * This can be used to avoid storing packet payload in memory or disk
+ */
+void
+capture_packet_free_frames(capture_packet_t *pkt);
+
 /**
  * @brief Set packet type
  */
 void
 capture_packet_set_type(capture_packet_t *packet, int type);
 
+/**
+ * @brief Set packet payload when it can not be get from packet
+ */
+void
+capture_packet_set_payload(capture_packet_t *packet, u_char *payload, uint32_t payload_len);
+
+/**
+ * @brief Getter for capture payload size
+ */
+uint32_t
+capture_packet_get_payload_len(capture_packet_t *packet);
+
+/**
+ * @brief Getter for capture payload pointer
+ */
+u_char *
+capture_packet_get_payload(capture_packet_t *packet);
+
+/**
+ * @brief Get The timestamp for a packet.
+ */
+struct timeval
+capture_packet_get_time(capture_packet_t *packet);
+
 /**
  * @brief Sorter by time for captured packets
  */
@@ -340,7 +469,7 @@ capture_close();
  * @brief Get datalink header size
  *
  */
-int
+int8_t
 datalink_size(int datalink);
 
 /**
@@ -355,7 +484,7 @@ dump_open(const char *dumpfile);
  * File must be previously opened with dump_open
  */
 void
-dump_packet(pcap_dumper_t *pd, const struct pcap_pkthdr *header, const u_char *packet);
+dump_packet(pcap_dumper_t *pd, const capture_packet_t *packet);
 
 /**
  * @brief Close a dump file
