@@ -34,6 +34,8 @@
 #include "capture.h"
 #include "capture_tls.h"
 #include "option.h"
+#include "util.h"
+#include "sip.h"
 
 struct SSLConnection *connections;
 
@@ -115,11 +117,9 @@ PRF(unsigned char *dest, int dlen, unsigned char *pre_master_secret, int plen, u
 }
 
 struct SSLConnection *
-tls_connection_create(struct in_addr caddr, u_short cport, struct in_addr saddr, u_short sport)
-{
+tls_connection_create(struct in_addr caddr, u_short cport, struct in_addr saddr, u_short sport) {
     struct SSLConnection *conn = NULL;
-    conn = malloc(sizeof(struct SSLConnection));
-    memset(conn, 0, sizeof(struct SSLConnection));
+    conn = sng_malloc(sizeof(struct SSLConnection));
 
     memcpy(&conn->client_addr, &caddr, sizeof(struct in_addr));
     memcpy(&conn->server_addr, &saddr, sizeof(struct in_addr));
@@ -167,7 +167,7 @@ tls_connection_destroy(struct SSLConnection *conn)
     // Deallocate connection memory
     SSL_CTX_free(conn->ssl_ctx);
     SSL_free(conn->ssl);
-    free(conn);
+    sng_free(conn);
 }
 
 /**
@@ -213,8 +213,7 @@ tls_connection_dir(struct SSLConnection *conn, struct in_addr addr, u_short port
 }
 
 struct SSLConnection*
-tls_connection_find(struct in_addr addr, u_short port)
-{
+tls_connection_find(struct in_addr addr, u_short port) {
     struct SSLConnection *conn;
 
     for (conn = connections; conn; conn = conn->next) {
@@ -226,30 +225,26 @@ tls_connection_find(struct in_addr addr, u_short port)
 }
 
 int
-tls_process_segment(const struct ip *ip, uint8 **out, uint32_t *outl)
+tls_process_segment(capture_packet_t *packet, struct tcphdr *tcp)
 {
     struct SSLConnection *conn;
-    struct tcphdr *tcp;
-    uint16_t tcp_size;
-    const uint8 *payload;
-    uint16_t ip_off;
-    uint8_t ip_frag;
-    uint16_t ip_frag_off;
-    int len;
+    const u_char *payload = capture_packet_get_payload(packet);
+    uint32_t size_payload = capture_packet_get_payload_len(packet);
+    uint8 *out;
+    uint32_t outl = packet->payload_len;
+    out = sng_malloc(outl);
+    struct in_addr ip_src, ip_dst;
+    u_short sport = packet->sport;
+    u_short dport = packet->dport;
 
-    // Process IP offset
-    ip_off = ntohs(ip->ip_off);
-    ip_frag = ip_off & (IP_MF | IP_OFFMASK);
-    ip_frag_off = (ip_frag) ? (ip_off & IP_OFFMASK) * 8 : 0;
-
-    // Get TCP header
-    tcp = (struct tcphdr *) ((uint8 *) ip + (ip->ip_hl * 4));
-    tcp_size = (ip_frag_off) ? 0 : (tcp->th_off * 4);
+    // Convert addresses
+    inet_pton(AF_INET, packet->ip_src, &ip_src);
+    inet_pton(AF_INET, packet->ip_dst, &ip_dst);
 
     // Try to find a session for this ip
-    if ((conn = tls_connection_find(ip->ip_src, tcp->th_sport))) {
+    if ((conn = tls_connection_find(ip_src, sport))) {
         // Update last connection direction
-        conn->direction = tls_connection_dir(conn, ip->ip_src, tcp->th_sport);
+        conn->direction = tls_connection_dir(conn, ip_src, sport);
 
         // Check current connection state
         switch (conn->state) {
@@ -266,10 +261,13 @@ tls_process_segment(const struct ip *ip, uint8 **out, uint32_t *outl)
             case TCP_STATE_ACK:
             case TCP_STATE_ESTABLISHED:
                 // Process data segment!
-                payload = (uint8 *) tcp + tcp_size;
-                len = ntohs(ip->ip_len) - (ip->ip_hl * 4) - tcp_size;
-                if (tls_process_record(conn, payload, len, out, outl) != 0)
-                    return 1;
+                if (tls_process_record(conn, payload, size_payload, &out, &outl) == 0) {
+                    if ((int32_t) outl > 0) {
+                        capture_packet_set_payload(packet, out, outl);
+                        capture_packet_set_type(packet, CAPTURE_PACKET_SIP_TLS);
+                        return 0;
+                    }
+                }
                 break;
             case TCP_STATE_FIN:
             case TCP_STATE_CLOSED:
@@ -280,10 +278,11 @@ tls_process_segment(const struct ip *ip, uint8 **out, uint32_t *outl)
     } else {
         if (tcp->th_flags & TH_SYN & ~TH_ACK) {
             // New connection, store it status and leave
-            tls_connection_create(ip->ip_src, tcp->th_sport, ip->ip_dst, tcp->th_dport);
+            tls_connection_create(ip_src, sport, ip_dst, dport);
         }
     }
 
+    sng_free(out);
     return 0;
 }
 
@@ -395,7 +394,7 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment)
                                     (unsigned char *) &conn->pre_master_secret,
                                     conn->server_private_key->pkey.rsa, RSA_PKCS1_PADDING);
 
-                unsigned char *seed = malloc(sizeof(struct Random) * 2);
+                unsigned char *seed = sng_malloc(sizeof(struct Random) * 2);
                 memcpy(seed, &conn->client_random, sizeof(struct Random));
                 memcpy(seed + sizeof(struct Random), &conn->server_random, sizeof(struct Random));
 
@@ -413,7 +412,7 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment)
                     (unsigned char *) "key expansion", seed, sizeof(struct Random) * 2);
 
                 // Done with the seed
-                free(seed);
+                sng_free(seed);
 
                 // Create Client decoder
                 EVP_CIPHER_CTX_init(&conn->client_cipher_ctx);
@@ -432,10 +431,10 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment)
             default:
                 if (conn->encrypted) {
                     // Encrypted Hanshake Message
-                    unsigned char *decoded = malloc(48);
+                    unsigned char *decoded = sng_malloc(48);
                     uint32_t decodedlen;
                     tls_process_record_data(conn, fragment, 48, &decoded, &decodedlen);
-                    free(decoded);
+                    sng_free(decoded);
                 }
                 break;
         }
@@ -451,7 +450,7 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment, cons
     EVP_CIPHER_CTX *evp;
     unsigned char pad;
     unsigned char *decoded;
-    int dlen;
+    uint32_t dlen;
 
     if (conn->direction == 0) {
         evp = &conn->client_cipher_ctx;
@@ -459,20 +458,20 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment, cons
         evp = &conn->server_cipher_ctx;
     }
 
-    decoded = malloc(len);
+    decoded = sng_malloc(len);
     EVP_Cipher(evp, decoded, (unsigned char *) fragment, len);
 
     // Get padding counter and remove from data
     pad = decoded[len - 1];
     dlen = (len - (pad + 1) - /* Trailing MAC */20);
 
-    if (dlen > 0) {
+    if ((int32_t)dlen > 0 && dlen <= *outl) {
         memcpy(*out, decoded, dlen);
         *outl = dlen;
     }
 
     // Clenaup decoded memory
-    free(decoded);
+    sng_free(decoded);
     return *outl;
 }
 
