@@ -33,7 +33,7 @@
 #include "filter.h"
 
 //! Storage of filter information
-filter_t filters[FILTER_COUNT] = {};
+filter_t filters[FILTER_COUNT] = { };
 
 int
 filter_set(int type, const char *expr)
@@ -54,7 +54,7 @@ filter_set(int type, const char *expr)
 
     // Remove previous value
     if (filters[type].expr) {
-        free(filters[type].expr);
+        sng_free(filters[type].expr);
         pcre_free(filters[type].regex);
     }
 
@@ -73,7 +73,7 @@ filter_set(int type, const char *expr)
 
     // Remove previous value
     if (filters[type].expr) {
-        free(filters[type].expr);
+        sng_free(filters[type].expr);
         regfree(&filters[type].regex);
     }
 
@@ -95,10 +95,13 @@ int
 filter_check_call(void *item)
 {
     int i;
-    const char *data;
-    char linetext[256];
+    char data[256];
     sip_call_t *call = (sip_call_t*) item;
 
+    // Dont filter calls without messages
+    if (call_msg_count(call) == 0)
+        return 0;
+
     // Filter for this call has already be processed
     if (call->filtered != -1)
         return (call->filtered == 0);
@@ -112,27 +115,29 @@ filter_check_call(void *item)
         if (!filters[i].expr)
             continue;
 
+        // Initialize
+        memset(data, 0, sizeof(data));
+
         // Get filtered field
         switch(i) {
             case FILTER_SIPFROM:
-                data = call_get_attribute(call, SIP_ATTR_SIPFROM);
+                call_get_attribute(call, SIP_ATTR_SIPFROM, data);
                 break;
             case FILTER_SIPTO:
-                data = call_get_attribute(call, SIP_ATTR_SIPTO);
+                call_get_attribute(call, SIP_ATTR_SIPTO, data);
                 break;
             case FILTER_SOURCE:
-                data = call_get_attribute(call, SIP_ATTR_SRC);
+                call_get_attribute(call, SIP_ATTR_SRC, data);
                 break;
             case FILTER_DESTINATION:
-                data = call_get_attribute(call, SIP_ATTR_DST);
+                call_get_attribute(call, SIP_ATTR_DST, data);
                 break;
             case FILTER_METHOD:
-                data = call_get_attribute(call, SIP_ATTR_METHOD);
+                call_get_attribute(call, SIP_ATTR_METHOD, data);
                 break;
             case FILTER_CALL_LIST:
                 // FIXME Maybe call should know hot to calculate this line
-                memset(linetext, 0, sizeof(linetext));
-                data = call_list_line_text(ui_get_panel(ui_find_by_type(PANEL_CALL_LIST)), call, linetext);
+                call_list_line_text(ui_get_panel(ui_find_by_type(PANEL_CALL_LIST)), call, data);
                 break;
             default:
                 // Unknown filter id
