@@ -33,8 +33,10 @@
 #include <ctype.h>
 #include <getopt.h>
 #include "option.h"
+#include "vector.h"
 #include "ui_manager.h"
 #include "capture.h"
+#include "capture_eep.h"
 #ifdef WITH_OPENSSL
 #include "capture_tls.h"
 #endif
@@ -63,6 +65,9 @@ usage()
            "    -i --icase\t\t Make <match expression> case insensitive\n"
            "    -v --invert\t\t Invert <match expression>\n"
            "    -N --no-interface\t Don't display sngrep interface, just capture\n"
+           "    -D --dump-config\t Print active configuration settings and exit\n"
+           "    -H --eep-send\t Homer sipcapture url (udp:X.X.X.X:XXXX)\n"
+           "    -L --eep-listen\t Listen for encapsulated packets (udp:X.X.X.X:XXXX)\n"
            "    -q --quiet\t\t Don't print captured dialogs in no interface mode\n"
 #ifdef WITH_OPENSSL
            "    -k --keyfile\t RSA private keyfile to decrypt captured packets\n"
@@ -88,7 +93,7 @@ version()
 #ifdef WITH_PCRE
            " * Compiled with Perl Compatible regular expressions support.\n"
 #endif
-#ifdef WITH_IPV6
+#ifdef USE_IPV6
            " * Compiled with IPv6 support.\n"
 #endif
            "\nWritten by Ivan Alonso [aka Kaian]\n",
@@ -104,12 +109,13 @@ int
 main(int argc, char* argv[])
 {
     int opt, idx, limit, only_calls, no_incomplete, i;
-    const char *device, *infile, *outfile;
+    const char *device, *outfile;
     char bpf[512];
     const char *keyfile;
     const char *match_expr;
     int match_insensitive = 0, match_invert = 0;
     int no_interface = 0, quiet = 0, rtp_capture = 0;
+    vector_t *infiles = vector_create(0, 1);
 
     // Program otptions
     static struct option long_options[] = {
@@ -125,6 +131,9 @@ main(int argc, char* argv[])
         { "icase", no_argument, 0, 'i' },
         { "invert", no_argument, 0, 'v' },
         { "no-interface", no_argument, 0, 'N' },
+        { "dump-config", no_argument, 0, 'D' },
+        { "eep-listen", required_argument, 0, 'L' },
+        { "eep-send", required_argument, 0, 'H' },
         { "quiet", no_argument, 0, 'q' },
     };
 
@@ -133,17 +142,16 @@ main(int argc, char* argv[])
 
     // Get initial values for configurable arguments
     device = setting_get_value(SETTING_CAPTURE_DEVICE);
-    infile = setting_get_value(SETTING_CAPTURE_INFILE);
     outfile = setting_get_value(SETTING_CAPTURE_OUTFILE);
     keyfile = setting_get_value(SETTING_CAPTURE_KEYFILE);
     limit = setting_get_intvalue(SETTING_CAPTURE_LIMIT);
     only_calls = setting_enabled(SETTING_SIP_CALLS);
     no_incomplete = setting_enabled(SETTING_SIP_NOINCOMPLETE);
-    rtp_capture = setting_enabled(SETTING_SIP_NOINCOMPLETE);
+    rtp_capture = setting_enabled(SETTING_CAPTURE_RTP);
 
     // Parse command line arguments
     opterr = 0;
-    char *options = "hVd:I:O:pqtW:k:crl:ivNq";
+    char *options = "hVd:I:O:pqtW:k:crl:ivNqDL:H:";
     while ((opt = getopt_long(argc, argv, options, long_options, &idx)) != -1) {
         switch (opt) {
             case 'h':
@@ -156,7 +164,7 @@ main(int argc, char* argv[])
                 device = optarg;
                 break;
             case 'I':
-                infile = optarg;
+                vector_append(infiles, optarg);
                 break;
             case 'O':
                 outfile = optarg;
@@ -172,9 +180,11 @@ main(int argc, char* argv[])
                 break;
             case 'c':
                 only_calls = 1;
+                setting_set_value(SETTING_SIP_CALLS, SETTING_ON);
                 break;
             case 'r':
                 rtp_capture = 1;
+                setting_set_value(SETTING_CAPTURE_RTP, SETTING_ON);
                 break;
             case 'i':
                 match_insensitive++;
@@ -184,15 +194,26 @@ main(int argc, char* argv[])
                 break;
             case 'N':
                 no_interface = 1;
+                setting_set_value(SETTING_CAPTURE_STORAGE, "none");
                 break;
             case 'q':
                 quiet = 1;
                 break;
-            // Dark options for dummy ones
+            case 'D':
+                key_bindings_dump();
+                settings_dump();
+                return 0;
+                // Dark options for dummy ones
             case 'p':
             case 't':
             case 'W':
                 break;
+            case 'L':
+                capture_eep_set_server_url(optarg);
+                break;
+            case 'H':
+                capture_eep_set_client_url(optarg);
+                break;
             case '?':
                 if (strchr(options, optopt)) {
                     fprintf(stderr, "-%c option requires an argument.\n", optopt);
@@ -228,19 +249,27 @@ main(int argc, char* argv[])
     sip_init(limit, only_calls, no_incomplete);
 
     // Set capture options
-    capture_set_opts(limit, rtp_capture);
+    capture_init(limit, rtp_capture);
+
+    // Initialize EEP if enabled
+    capture_eep_init();
 
     // If we have an input file, load it
-    if (infile) {
-        // Try to load file
-        if (capture_offline(infile, outfile) != 0)
-            return 1;
+    if (vector_count(infiles)) {
+        for (i = 0; i < vector_count(infiles); i++) {
+            // Try to load file
+            if (capture_offline(vector_item(infiles, i), outfile) != 0)
+                return 1;
+        }
     } else {
         // Check if all capture data is valid
         if (capture_online(device, outfile) != 0)
             return 1;
     }
 
+    // Remove Input files vector
+    vector_destroy(infiles);
+
     // More arguments pending!
     if (argv[optind]) {
         // Assume first pending argument is  match expression
@@ -302,8 +331,8 @@ main(int argc, char* argv[])
             printf("\rDialog count: %d\n", sip_calls_count());
     }
 
-    // Close pcap handler
-    capture_close();
+    // Capture deinit
+    capture_deinit();
 
     // Deinitialize interface
     ncurses_deinit();
