@@ -41,6 +41,9 @@
 #ifndef __SNGREP_SETTING_H
 #define __SNGREP_SETTING_H
 
+//! Max setting value
+#define MAX_SETTING_LEN   80
+
 //! Shorter declarartion of setting_option struct
 typedef struct setting_option setting_t;
 
@@ -50,11 +53,18 @@ typedef struct setting_option setting_t;
 #define SETTING_ENUM_BACKGROUND  (const char *[]){ "dark" , "default", NULL }
 #define SETTING_ENUM_COLORMODE   (const char *[]){ "request", "cseq", "callid", NULL }
 #define SETTING_ENUM_HIGHLIGHT   (const char *[]){ "bold", "reverse", "reversebold", NULL }
-#define SETTING_ENUM_SDP_INFO    (const char *[]){ "off", "first", "full", NULL}
+#define SETTING_ENUM_SDP_INFO    (const char *[]){ "off", "first", "full", "compressed", NULL}
+#define SETTING_ENUM_STORAGE     (const char *[]){ "none", "memory", NULL }
+
+//! Other useful defines
+#define SETTING_ON  "on"
+#define SETTING_OFF "off"
+#define SETTING_YES "yes"
+#define SETTING_NO  "no"
+
 
 //! Available setting Options
-enum setting_id
-{
+enum setting_id {
     SETTING_BACKGROUND = 0,
     SETTING_COLORMODE,
     SETTING_SYNTAX,
@@ -65,17 +75,17 @@ enum setting_id
     SETTING_CAPTURE_LIMIT,
     SETTING_CAPTURE_LOOKUP,
     SETTING_CAPTURE_DEVICE,
-    SETTING_CAPTURE_INFILE,
     SETTING_CAPTURE_OUTFILE,
     SETTING_CAPTURE_KEYFILE,
     SETTING_CAPTURE_RTP,
+    SETTING_CAPTURE_STORAGE,
     SETTING_SIP_NOINCOMPLETE,
     SETTING_SIP_CALLS,
     SETTING_SAVEPATH,
     SETTING_DISPLAY_HOST,
     SETTING_DISPLAY_ALIAS,
-    SETTING_CL_FILTER,
     SETTING_CL_SCROLLSTEP,
+    SETTING_CL_COLORATTR,
     SETTING_CF_FORCERAW,
     SETTING_CF_RAWMINWIDTH,
     SETTING_CF_RAWFIXEDWIDTH,
@@ -88,12 +98,19 @@ enum setting_id
     SETTING_CF_DELTA,
     SETTING_CR_SCROLLSTEP,
     SETTING_FILTER_METHODS,
-    SETTING_COUNT,
+    SETTING_EEP_SEND,
+    SETTING_EEP_SEND_ADDR,
+    SETTING_EEP_SEND_PORT,
+    SETTING_EEP_SEND_PASS,
+    SETTING_EEP_LISTEN,
+    SETTING_EEP_LISTEN_ADDR,
+    SETTING_EEP_LISTEN_PORT,
+    SETTING_EEP_LISTEN_PASS,
+    SETTING_COUNT
 };
 
 //! Available setting formats
-enum setting_fmt
-{
+enum setting_fmt {
     SETTING_FMT_STRING = 0,
     SETTING_FMT_NUMBER,
     SETTING_FMT_ENUM,
@@ -110,11 +127,9 @@ struct setting_option {
     //! Setting format
     enum setting_fmt fmt;
     //! Value of the setting
-    char *value;
+    char value[MAX_SETTING_LEN];
     //! Compa separated valid values
     const char **valuelist;
-    //! Setting set by user config
-    int user;
 };
 
 setting_t *
@@ -171,4 +186,14 @@ setting_toggle(int id);
 const char *
 setting_enum_next(int id, const char *value);
 
+/**
+ * @brief Dump configuration settings
+ *
+ * This function will print to stdout configuration settings
+ * after reading system/local/user resource files (in that order).
+ *
+ */
+void
+settings_dump();
+
 #endif /* __SNGREP_SETTING_H */
