@@ -24,8 +24,6 @@
  * @author Ivan Alonso [aka Kaian] <kaian@irontec.com>
  *
  * @brief Source of functions defined in sip.h
- *
- * @todo Replace structures for their typedef shorter names
  */
 #include "config.h"
 #include <stdlib.h>
@@ -47,7 +45,97 @@
  * this awesome structure, so, keep it thread-safe.
  */
 sip_call_list_t calls =
-    { 0 };
+{ 0 };
+
+/* @brief list of methods and responses */
+sip_code_t sip_codes[] = {
+    { SIP_METHOD_REGISTER,  "REGISTER" },
+    { SIP_METHOD_INVITE,    "INVITE" },
+    { SIP_METHOD_SUBSCRIBE, "SUBSCRIBE" },
+    { SIP_METHOD_NOTIFY,    "NOTIFY" },
+    { SIP_METHOD_OPTIONS,   "OPTIONS" },
+    { SIP_METHOD_PUBLISH,   "PUBLISH" },
+    { SIP_METHOD_MESSAGE,   "MESSAGE" },
+    { SIP_METHOD_CANCEL,    "CANCEL" },
+    { SIP_METHOD_BYE,       "BYE" },
+    { SIP_METHOD_ACK,       "ACK" },
+    { SIP_METHOD_PRACK,     "PRACK" },
+    { SIP_METHOD_INFO,      "INFO" },
+    { SIP_METHOD_REFER,     "REFER" },
+    { SIP_METHOD_UPDATE,    "UPDATE" },
+    { 100, "100 Trying" },
+    { 180, "180 Ringing" },
+    { 181, "181 Call is Being Forwarded" },
+    { 182, "182 Queued" },
+    { 183, "183 Session Progress" },
+    { 199, "199 Early Dialog Terminated" },
+    { 200, "200 OK" },
+    { 202, "202 Accepted" },
+    { 204, "204 No Notification" },
+    { 300, "300 Multiple Choices" },
+    { 301, "301 Moved Permanently" },
+    { 302, "302 Moved Temporarily" },
+    { 305, "305 Use Proxy" },
+    { 380, "380 Alternative Service" },
+    { 400, "400 Bad Request" },
+    { 401, "401 Unauthorized" },
+    { 402, "402 Payment Required" },
+    { 403, "403 Forbidden" },
+    { 404, "404 Not Found" },
+    { 405, "405 Method Not Allowed" },
+    { 406, "406 Not Acceptable" },
+    { 407, "407 Proxy Authentication Required" },
+    { 408, "408 Request Timeout" },
+    { 409, "409 Conflict" },
+    { 410, "410 Gone" },
+    { 411, "411 Length Required" },
+    { 412, "412 Conditional Request Failed" },
+    { 413, "413 Request Entity Too Large" },
+    { 414, "414 Request-URI Too Long" },
+    { 415, "415 Unsupported Media Type" },
+    { 416, "416 Unsupported URI Scheme" },
+    { 417, "417 Unknown Resource-Priority" },
+    { 420, "420 Bad Extension" },
+    { 421, "421 Extension Required" },
+    { 422, "422 Session Interval Too Small" },
+    { 423, "423 Interval Too Brief" },
+    { 424, "424 Bad Location Information" },
+    { 428, "428 Use Identity Header" },
+    { 429, "429 Provide Referrer Identity" },
+    { 430, "430 Flow Failed" },
+    { 433, "433 Anonymity Disallowed" },
+    { 436, "436 Bad Identity-Info" },
+    { 437, "437 Unsupported Certificate" },
+    { 438, "438 Invalid Identity Header" },
+    { 439, "439 First Hop Lacks Outbound Support" },
+    { 470, "470 Consent Needed" },
+    { 480, "480 Temporarily Unavailable" },
+    { 481, "481 Call/Transaction Does Not Exist" },
+    { 482, "482 Loop Detected." },
+    { 483, "483 Too Many Hops" },
+    { 484, "484 Address Incomplete" },
+    { 485, "485 Ambiguous" },
+    { 486, "486 Busy Here" },
+    { 487, "487 Request Terminated" },
+    { 488, "488 Not Acceptable Here" },
+    { 489, "489 Bad Event" },
+    { 491, "491 Request Pending" },
+    { 493, "493 Undecipherable" },
+    { 494, "494 Security Agreement Required" },
+    { 500, "500 Server Internal Error" },
+    { 501, "501 Not Implemented" },
+    { 502, "502 Bad Gateway" },
+    { 503, "503 Service Unavailable" },
+    { 504, "504 Server Time-out" },
+    { 505, "505 Version Not Supported" },
+    { 513, "513 Message Too Large" },
+    { 580, "580 Precondition Failure" },
+    { 600, "600 Busy Everywhere" },
+    { 603, "603 Decline" },
+    { 604, "604 Does Not Exist Anywhere" },
+    { 606, "606 Not Acceptable" },
+    { -1 , NULL },
+};
 
 void
 sip_init(int limit, int only_calls, int no_incomplete)
@@ -62,20 +150,11 @@ sip_init(int limit, int only_calls, int no_incomplete)
     // Create a vector to store calls
     calls.list = vector_create(200, 50);
     vector_set_destroyer(calls.list, call_destroyer);
+    calls.active = vector_create(10, 10);
 
     // Create hash table for callid search
     hcreate(calls.limit);
 
-    // Initialize calls lock
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init(&attr);
-#if defined(PTHREAD_MUTEX_RECURSIVE) || defined(__FreeBSD__) || defined(BSD) || defined (__OpenBSD__)
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
-#else
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
-#endif
-    pthread_mutex_init(&calls.lock, &attr);
-
     // Initialize payload parsing regexp
     match_flags = REG_EXTENDED | REG_ICASE | REG_NEWLINE;
     regcomp(&calls.reg_method, "^([a-zA-Z]+) sip:[^ ]+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
@@ -92,10 +171,11 @@ sip_deinit()
 {
     // Remove all calls
     sip_calls_clear();
-    // Remove calls vector
-    vector_destroy(calls.list);
     // Remove Call-id hash table
     hdestroy();
+    // Remove calls vector
+    vector_destroy(calls.list);
+    vector_destroy(calls.active);
     // Deallocate regular expressions
     regfree(&calls.reg_method);
     regfree(&calls.reg_callid);
@@ -104,8 +184,6 @@ sip_deinit()
     regfree(&calls.reg_cseq);
     regfree(&calls.reg_from);
     regfree(&calls.reg_to);
-    // Remove calls mutex
-    pthread_mutex_destroy(&calls.lock);
 }
 
 
@@ -116,35 +194,66 @@ sip_get_callid(const char* payload, char *callid)
 
     // Try to get Call-ID from payload
     if (regexec(&calls.reg_callid, payload, 3, pmatch, 0) == 0) {
-        // Allocate memory for Call-Id (caller MUST free it)
-        memset(callid, 0, pmatch[2].rm_eo - pmatch[2].rm_so + 1);
         // Copy the matching part of payload
-        strncpy(callid, payload + pmatch[2].rm_so, pmatch[2].rm_eo - pmatch[2].rm_so);
+        strncpy(callid, payload + pmatch[2].rm_so, (int) pmatch[2].rm_eo - pmatch[2].rm_so);
     }
 
     return callid;
 }
 
+char *
+sip_get_xcallid(const char *payload, char *xcallid)
+{
+    regmatch_t pmatch[3];
+
+    // Try to get X-Call-ID from payload
+    if (regexec(&calls.reg_xcallid, (const char *)payload, 3, pmatch, 0) == 0) {
+        strncpy(xcallid, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
+    }
+
+    return xcallid;
+}
+
 sip_msg_t *
-sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const char* dst, u_short dport, u_char *payload)
+sip_check_packet(capture_packet_t *packet)
 {
     ENTRY entry;
     sip_msg_t *msg;
     sip_call_t *call;
-    int call_idx;
-    char callid[1024];
+    char callid[1024], xcallid[1024];
+    const char *src, *dst;
+    u_short sport, dport;
     char msg_src[ADDRESSLEN];
     char msg_dst[ADDRESSLEN];
+    u_char payload[MAX_SIP_PAYLOAD];
+
+    // Max SIP payload allowed
+    if (packet->payload_len > MAX_SIP_PAYLOAD)
+        return NULL;
+
+    // Get Addresses from packet
+    src = packet->ip_src;
+    dst = packet->ip_dst;
+    sport = packet->sport;
+    dport = packet->dport;
+
+    // Initialize local variables
+    memset(callid, 0, sizeof(callid));
+    memset(xcallid, 0, sizeof(xcallid));
+    memset(msg_src, 0, sizeof(msg_src));
+    memset(msg_dst, 0, sizeof(msg_dst));
+
+    // Get payload from packet(s)
+    memset(payload, 0, MAX_SIP_PAYLOAD);
+    memcpy(payload, capture_packet_get_payload(packet), capture_packet_get_payload_len(packet));
 
     // Get the Call-ID of this message
-    if (!sip_get_callid((const char*) payload, callid)) {
+    if (!sip_get_callid((const char*) payload, callid))
         return NULL;
-    }
 
     // Create a new message from this data
-    if (!(msg = msg_create((const char*) payload))) {
+    if (!(msg = msg_create((const char*) payload)))
         return NULL;
-    }
 
     // Get Method and request for the following checks
     // There is no need to parse all payload at this point
@@ -155,11 +264,6 @@ sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const
         return NULL;
     }
 
-    // If payload is encrypted, dup payload
-    if (packet->type == CAPTURE_PACKET_SIP_TLS)
-        msg->payload = (u_char *)strdup((const char *)payload);
-
-    pthread_mutex_lock(&calls.lock);
     // Find the call for this msg
     if (!(call = sip_find_by_callid(callid))) {
 
@@ -176,26 +280,21 @@ sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const
         if (calls.ignore_incomplete && msg->reqresp > SIP_METHOD_MESSAGE)
             goto skip_message;
 
-        // Check if this message is ignored by configuration directive
-        if (sip_check_msg_ignore(msg))
-            goto skip_message;
+        // Get the Call-ID of this message
+        sip_get_xcallid((const char*) payload, xcallid);
 
         // Create the call if not found
-        if (!(call = call_create(callid)))
+        if (!(call = call_create(callid, xcallid)))
             goto skip_message;
 
         // Store this call in hash table
-        entry.key = strdup(call_get_attribute(call, SIP_ATTR_CALLID));
+        entry.key = (char *) call->callid;
         entry.data = (void *) call;
         hsearch(entry, ENTER);
 
         // Append this call to the call list
-        pthread_mutex_lock(&calls.lock);
-        call_idx = vector_append(calls.list, call);
-        pthread_mutex_unlock(&calls.lock);
-
-        // Store current call Index
-        call_set_attribute(call, SIP_ATTR_CALLINDEX, "%d", call_idx);
+        vector_append(calls.list, call);
+        call->index = vector_count(calls.list);
     }
 
     // Store sorce address. Prefix too long IPv6 addresses with two dots
@@ -212,32 +311,41 @@ sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const
         strcpy(msg_dst, dst);
     }
 
-    // Set Source and Destination attributes
-    msg_set_attribute(msg, SIP_ATTR_SRC, "%s:%u", msg_src, sport);
-    msg_set_attribute(msg, SIP_ATTR_DST, "%s:%u", msg_dst, dport);
+    // At this point we know we're handling an interesting SIP Packet
+    msg->packet = packet;
+
+    // Always parse first call message
+    if (call_msg_count(call) == 0) {
+        // Parse SIP payload
+        sip_parse_msg_payload(msg, payload);
+    }
 
-    // Add this SIP packet to the message
-    msg_add_packet(msg, packet);
     // Add the message to the call
     call_add_message(call, msg);
 
-    // Always parse first call message
-    if (call_msg_count(call) == 1 || call_is_invite(call)) {
-        // Parse SIP payload
-        sip_parse_msg_payload(msg, payload);
+    if (call_is_invite(call)) {
         // Parse media data
         sip_parse_msg_media(msg, payload);
         // Update Call State
         call_update_state(call, msg);
+        // Check if this call should be in active call list
+        if (call_is_active(call)) {
+            if (vector_index(calls.active, call) == -1) {
+                vector_append(calls.active, call);
+            }
+        } else {
+            if (vector_index(calls.active, call) != -1) {
+                vector_remove(calls.active, call);
+            }
+        }
     }
-    pthread_mutex_unlock(&calls.lock);
+
     // Return the loaded message
     return msg;
 
 skip_message:
     // Deallocate message memory
     msg_destroy(msg);
-    pthread_mutex_unlock(&calls.lock);
     return NULL;
 
 }
@@ -254,20 +362,29 @@ sip_calls_iterator()
     return vector_iterator(calls.list);
 }
 
+vector_iter_t
+sip_active_calls_iterator()
+{
+    return vector_iterator(calls.active);
+}
+
 void
 sip_calls_stats(int *total, int *displayed)
 {
     vector_iter_t it = vector_iterator(calls.list);
 
-    pthread_mutex_lock(&calls.lock);
     // Total number of calls without filtering
     *total = vector_iterator_count(&it);
     // Total number of calls after filtering
     vector_iterator_set_filter(&it, filter_check_call);
     *displayed = vector_iterator_count(&it);
-    pthread_mutex_unlock(&calls.lock);
 }
 
+sip_call_t *
+sip_find_by_index(int index)
+{
+    return vector_item(calls.list, index);
+}
 
 sip_call_t *
 sip_find_by_callid(const char *callid)
@@ -284,17 +401,16 @@ sip_find_by_callid(const char *callid)
 sip_call_t *
 sip_find_by_xcallid(const char *xcallid)
 {
-    const char *cur_xcallid;
     sip_call_t *cur;
-    int i;
+    vector_iter_t it = vector_iterator(calls.list);
 
-    for (i=0; i < vector_count(calls.list); i++) {
-        cur = vector_item(calls.list, i);
-        cur_xcallid = call_get_attribute(cur, SIP_ATTR_XCALLID);
-        if (cur_xcallid && !strcmp(cur_xcallid, xcallid)) {
+    // Find the call with the given X-Call-Id
+    while ((cur = vector_iterator_next(&it))) {
+        if (strlen(cur->xcallid) && !strcmp(cur->xcallid, xcallid)) {
             return cur;
         }
     }
+    // None found
     return NULL;
 }
 
@@ -303,13 +419,11 @@ sip_call_t *
 call_get_xcall(sip_call_t *call)
 {
     sip_call_t *xcall;
-    pthread_mutex_lock(&calls.lock);
-    if (call_get_attribute(call, SIP_ATTR_XCALLID)) {
-        xcall = sip_find_by_callid(call_get_attribute(call, SIP_ATTR_XCALLID));
+    if (strlen(call->xcallid)) {
+        xcall = sip_find_by_callid(call->xcallid);
     } else {
-        xcall = sip_find_by_xcallid(call_get_attribute(call, SIP_ATTR_CALLID));
+        xcall = sip_find_by_xcallid(call->callid);
     }
-    pthread_mutex_unlock(&calls.lock);
     return xcall;
 }
 
@@ -326,15 +440,12 @@ sip_get_msg_reqresp(sip_msg_t *msg, const u_char *payload)
 
         // Method & CSeq
         if (regexec(&calls.reg_method, (const char *)payload, 2, pmatch, 0) == 0) {
-            sprintf(reqresp, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
-            msg_set_attribute(msg, SIP_ATTR_METHOD, reqresp);
+            sprintf(reqresp, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
         }
 
         // Response code
         if (regexec(&calls.reg_response, (const char *)payload, 3, pmatch, 0) == 0) {
-            msg_set_attribute(msg, SIP_ATTR_METHOD, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so,
-                              payload + pmatch[1].rm_so);
-            sprintf(reqresp, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
+            sprintf(reqresp, "%.*s", (int)(pmatch[2].rm_eo - pmatch[2].rm_so), payload + pmatch[2].rm_so);
         }
 
         // Get Request/Response Code
@@ -344,6 +455,27 @@ sip_get_msg_reqresp(sip_msg_t *msg, const u_char *payload)
     return msg->reqresp;
 }
 
+const char *
+sip_get_response_str(sip_msg_t *msg, char *out)
+{
+    regmatch_t pmatch[3];
+    const char *payload;
+
+    // If not already parsed
+    if (msg_is_request(msg))
+        return NULL;
+
+    // Get message payload
+    payload = msg_get_payload(msg);
+
+    // Response code (full text)
+    if (regexec(&calls.reg_response, payload, 3, pmatch, 0) == 0) {
+        sprintf(out, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
+    }
+
+    return out;
+}
+
 sip_msg_t *
 sip_parse_msg(sip_msg_t *msg)
 {
@@ -357,50 +489,24 @@ int
 sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
 {
     regmatch_t pmatch[4];
-    char date[12], time[20], cseq[11];
+    char cseq[11];
 
     // CSeq
     if (regexec(&calls.reg_cseq, (char*)payload, 2, pmatch, 0) == 0) {
-        sprintf(cseq, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
+        sprintf(cseq, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
         msg->cseq = atoi(cseq);
     }
 
-    // X-Call-Id
-    if (regexec(&calls.reg_xcallid, (const char *)payload, 3, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_XCALLID, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
-                          payload + pmatch[2].rm_so);
-    }
-
     // From
     if (regexec(&calls.reg_from, (const char *)payload, 4, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SIPFROM, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
-                          payload + pmatch[2].rm_so);
-        msg_set_attribute(msg, SIP_ATTR_SIPFROMUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
-                          payload + pmatch[3].rm_so);
+        msg->sip_from = sng_malloc((int)pmatch[2].rm_eo - pmatch[2].rm_so + 1);
+        strncpy(msg->sip_from, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
     }
 
     // To
     if (regexec(&calls.reg_to, (const char *)payload, 4, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SIPTO, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
-                          payload + pmatch[2].rm_so);
-        msg_set_attribute(msg, SIP_ATTR_SIPTOUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
-                          payload + pmatch[3].rm_so);
-    }
-
-    // Set message Date and Time attribute
-    msg_set_attribute(msg, SIP_ATTR_DATE, timeval_to_date(msg_get_time(msg), date));
-    msg_set_attribute(msg, SIP_ATTR_TIME, timeval_to_time(msg_get_time(msg), time));
-
-    capture_packet_t *packet = vector_first(msg->packets);
-    // Store Transport attribute
-    if (packet->type == CAPTURE_PACKET_SIP_UDP) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "UDP");
-    } else if (packet->type == CAPTURE_PACKET_SIP_TCP) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TCP");
-    } else if (packet->type == CAPTURE_PACKET_SIP_TLS) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TLS");
-    } else if (packet->type == CAPTURE_PACKET_SIP_WS) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "WS");
+        msg->sip_to = sng_malloc((int)pmatch[2].rm_eo - pmatch[2].rm_so + 1);
+        strncpy(msg->sip_to, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
     }
 
     return 0;
@@ -414,8 +520,8 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
     char media_type[15] = { };
     char media_format[30] = { };
     int media_port;
-    int media_fmt_pref;
-    int media_fmt_code;
+    u_int media_fmt_pref;
+    u_int media_fmt_code;
     sdp_media_t *media = NULL;
     char *payload2, *tofree, *line;
 
@@ -427,13 +533,13 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
     while ((line = strsep(&payload2, "\r\n")) != NULL) {
         // Check if we have a media string
         if (!strncmp(line, "m=", 2)) {
-            if (sscanf(line, "m=%s %d RTP/AVP %d", media_type, &media_port, &media_fmt_pref) == 3) {
+            if (sscanf(line, "m=%s %d RTP/%*s %u", media_type, &media_port, &media_fmt_pref) == 3) {
                 // Create a new media structure for this message
                 if ((media = media_create(msg))) {
                     media_set_type(media, media_type);
                     media_set_port(media, media_port);
                     media_set_address(media, media_address);
-                    media_set_format_code(media, media_fmt_pref);
+                    media_set_prefered_format(media, media_fmt_pref);
                     msg_add_media(msg, media);
 
                     /**
@@ -452,40 +558,31 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
         // Check if we have a connection string
         if (!strncmp(line, "c=", 2)) {
             if (sscanf(line, "c=IN IP4 %s", media_address) && media) {
-                    media_set_address(media, media_address);
+                media_set_address(media, media_address);
             }
         }
 
         // Check if we have attribute format string
         if (!strncmp(line, "a=rtpmap:", 9)) {
-            if (sscanf(line, "a=rtpmap:%d %[^ ]", &media_fmt_code, media_format)) {
-                if (media && media_fmt_pref == media_fmt_code) {
-                    media_set_format(media, media_format);
-                }
+            if (media && sscanf(line, "a=rtpmap:%u %[^ ]", &media_fmt_code, media_format)) {
+                media_add_format(media, media_fmt_code, media_format);
             }
         }
-    }
-    free(tofree);
 
-    // If message has media
-    if ((media = vector_first(msg->medias))) {
-        msg_set_attribute(msg, SIP_ATTR_SDP_ADDRESS, media_get_address(media));
-        msg_set_attribute(msg, SIP_ATTR_SDP_PORT, "%d", media_get_port(media));
     }
-
+    sng_free(tofree);
 }
 
 
 void
 sip_calls_clear()
 {
-    pthread_mutex_lock(&calls.lock);
-    // Remove all items from vector
-    vector_clear(calls.list);
     // Create again the callid hash table
     hdestroy();
     hcreate(calls.limit);
-    pthread_mutex_unlock(&calls.lock);
+    // Remove all items from vector
+    vector_clear(calls.list);
+    vector_clear(calls.active);
 }
 
 int
@@ -512,7 +609,7 @@ sip_set_match_expression(const char *expr, int insensitive, int invert)
 
     // Case insensitive requested
     if (insensitive)
-    cflags |= REG_ICASE;
+        cflags |= REG_ICASE;
 
     // Check the expresion is a compilable regexp
     return regcomp(&calls.match_regex, expr, cflags) != 0;
@@ -539,57 +636,15 @@ sip_check_match_expression(const char *payload)
 #endif
 }
 
-
-int
-sip_check_msg_ignore(sip_msg_t *msg)
+const char *
+sip_method_str(int method)
 {
     int i;
-    sip_attr_hdr_t *header;
 
-    // Check if an ignore option exists
-    for (i = 0; i < SIP_ATTR_COUNT; i++) {
-        header = sip_attr_get_header(i);
-        if (is_ignored_value(header->name, msg_get_attribute(msg, header->id))) {
-            return 1;
-        }
-    }
-    return 0;
-}
-
-const char *
-sip_method_str(enum sip_methods method)
-{
-    switch (method) {
-        case SIP_METHOD_REGISTER:
-            return "REGISTER";
-        case SIP_METHOD_INVITE:
-            return "INVITE";
-        case SIP_METHOD_SUBSCRIBE:
-            return "SUBSCRIBE";
-        case SIP_METHOD_NOTIFY:
-            return "NOTIFY";
-        case SIP_METHOD_OPTIONS:
-            return "OPTIONS";
-        case SIP_METHOD_PUBLISH:
-            return "PUBLISH";
-        case SIP_METHOD_MESSAGE:
-            return "MESSAGE";
-        case SIP_METHOD_CANCEL:
-            return "CANCEL";
-        case SIP_METHOD_BYE:
-            return "BYE";
-        case SIP_METHOD_ACK:
-            return "ACK";
-        case SIP_METHOD_PRACK:
-            return "PRACK";
-        case SIP_METHOD_INFO:
-            return "INFO";
-        case SIP_METHOD_REFER:
-            return "REFER";
-        case SIP_METHOD_UPDATE:
-            return "UPDATE";
-        case SIP_METHOD_SENTINEL:
-            return "";
+    // Standard method
+    for (i = 0; sip_codes[i].id > 0; i++) {
+        if (method == sip_codes[i].id)
+            return sip_codes[i].text;
     }
     return NULL;
 }
@@ -598,25 +653,47 @@ int
 sip_method_from_str(const char *method)
 {
     int i;
-    for (i = 1; i < SIP_METHOD_SENTINEL; i++)
-        if (!strcmp(method, sip_method_str(i)))
-            return i;
+
+    // Standard method
+    for (i = 0; sip_codes[i].id > 0; i++) {
+        if (!strcmp(method, sip_codes[i].text))
+            return sip_codes[i].id;
+    }
     return atoi(method);
 }
 
+const char *
+sip_transport_str(int transport)
+{
+    switch(transport)
+    {
+        case CAPTURE_PACKET_SIP_UDP:
+            return "UDP";
+        case CAPTURE_PACKET_SIP_TCP:
+            return "TCP";
+        case CAPTURE_PACKET_SIP_TLS:
+            return "TLS";
+        case CAPTURE_PACKET_SIP_WS:
+            return "WS";
+        case CAPTURE_PACKET_SIP_WSS:
+            return "WSS";
+    }
+    return "";
+}
+
 char *
 sip_get_msg_header(sip_msg_t *msg, char *out)
 {
-    // Source and Destination address
-    char from_addr[80], to_addr[80];
+    char from_addr[80], to_addr[80], time[80], date[80];
 
-    // We dont use Message attributes here because it contains truncated data
-    // This should not overload too much as all results should be already cached
-    sprintf(from_addr, "%s", sip_address_port_format(SRC(msg)));
-    sprintf(to_addr, "%s", sip_address_port_format(DST(msg)));
+    // Source and Destination address
+    msg_get_attribute(msg, SIP_ATTR_DATE, date);
+    msg_get_attribute(msg, SIP_ATTR_TIME, time);
+    msg_get_attribute(msg, SIP_ATTR_SRC, from_addr);
+    msg_get_attribute(msg, SIP_ATTR_DST, to_addr);
 
     // Get msg header
-    sprintf(out, "%s %s %s -> %s", DATE(msg), TIME(msg), from_addr, to_addr);
+    sprintf(out, "%s %s %s -> %s", date, time, from_addr, to_addr);
     return out;
 }
 
