@@ -40,8 +40,40 @@
 #include "sip_call.h"
 #include "vector.h"
 
+#define MAX_SIP_PAYLOAD 10240
+
 //! Shorter declaration of sip_call_list structure
 typedef struct sip_call_list sip_call_list_t;
+//! Shorter declaration of sip codes structure
+typedef struct sip_code sip_code_t;
+
+//! SIP Methods
+enum sip_methods {
+    SIP_METHOD_REGISTER = 1,
+    SIP_METHOD_INVITE,
+    SIP_METHOD_SUBSCRIBE,
+    SIP_METHOD_NOTIFY,
+    SIP_METHOD_OPTIONS,
+    SIP_METHOD_PUBLISH,
+    SIP_METHOD_MESSAGE,
+    SIP_METHOD_CANCEL,
+    SIP_METHOD_BYE,
+    SIP_METHOD_ACK,
+    SIP_METHOD_PRACK,
+    SIP_METHOD_INFO,
+    SIP_METHOD_REFER,
+    SIP_METHOD_UPDATE,
+};
+
+/**
+ * @brief Different Request/Response codes in SIP Protocol
+ */
+struct sip_code
+{
+    int id;
+    const char *text;
+};
+
 
 /**
  * @brief call structures head list
@@ -51,6 +83,8 @@ typedef struct sip_call_list sip_call_list_t;
 struct sip_call_list {
     //! List of all captured calls
     vector_t *list;
+    //! List of active captured calls
+    vector_t *active;
 
     // Max call limit
     int limit;
@@ -78,9 +112,6 @@ struct sip_call_list {
     regex_t reg_cseq;
     regex_t reg_from;
     regex_t reg_to;
-
-    // Warranty thread-safe access to the calls list
-    pthread_mutex_t lock;
 };
 
 /**
@@ -111,20 +142,29 @@ sip_deinit();
 char *
 sip_get_callid(const char* payload, char *callid);
 
+/**
+ * @brief Parses X-Call-ID header of a SIP message payload
+ *
+ * Mainly used to check if a payload contains a xcallid.
+ *
+ * @param payload SIP message payload
+ * @paramx callid Character array to store callid
+ * @return xcallid parsed from Call-ID header
+ */
+char *
+sip_get_xcallid(const char* payload, char *xcallid);
+
 /**
  * @brief Loads a new message from raw header/payload
  *
  * Use this function to convert raw data into call and message
  * structures. This is mainly used to load data from a file or
  *
- * @todo This functions should stop using ngrep header format
- *
- * @param header Raw ngrep header
- * @param payload Raw ngrep payload
+ * @param packet Packet structure pointer
  * @return a SIP msg structure pointer
  */
 sip_msg_t *
-sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const char* dst, u_short dport, u_char *payload);
+sip_check_packet(capture_packet_t *packet);
 
 /**
  * @brief Getter for calls linked list size
@@ -140,6 +180,16 @@ sip_calls_count();
 vector_iter_t
 sip_calls_iterator();
 
+/**
+ * @brief Return an iterator of call list
+ *
+ * We consider 'active' calls those that are willing to have
+ * an rtp stream that will receive new packets.
+ *
+ */
+vector_iter_t
+sip_active_calls_iterator();
+
 /**
  * @brief Return stats from call list
  *
@@ -149,6 +199,16 @@ sip_calls_iterator();
 void
 sip_calls_stats(int *total, int *displayed);
 
+
+/**
+ * @brief Find a call structure in calls linked list given a call index
+ *
+ * @param index Position of the call in the calls vector
+ * @return pointer to the sip_call structure found or NULL
+ */
+sip_call_t *
+sip_find_by_index(int index);
+
 /**
  * @brief Find a call structure in calls linked list given an callid
  *
@@ -190,6 +250,14 @@ sip_calls_clear();
 int
 sip_get_msg_reqresp(sip_msg_t *msg, const u_char *payload);
 
+/**
+ * @brief Get full Response code (including text)
+ *
+ *
+ */
+const char *
+sip_get_response_str(sip_msg_t *msg, char *out);
+
 /**
  * @brief Parse SIP Message payload if not parsed
  *
@@ -246,23 +314,20 @@ sip_set_match_expression(const char *expr, int insensitive, int invert);
 int
 sip_check_match_expression(const char *payload);
 
-/**
- * @brief Check if this msg is affected by filters
- *
- * @param call Message to check
- * @return 1 if msg is filtered, 0 otherwise
- */
-int
-sip_check_msg_ignore(struct sip_msg *msg);
-
 /**
  * @brief Get String value for a Method
  *
- * @param method One of the methods defined in @sip_methods
+ * @param method One of the methods defined in @sip_codes
  * @return a string representing the method text
  */
 const char *
-sip_method_str(enum sip_methods method);
+sip_method_str(int method);
+
+/*
+ * @brief Get String value of Transport
+ */
+const char *
+sip_transport_str(int transport);
 
 /**
  * @brief Converts Request Name or Response code to number
