@@ -24,10 +24,8 @@
  * @author Ivan Alonso [aka Kaian] <kaian@irontec.com>
  *
  * @brief Source of functions defined in ui_call_flow.h
- *
- * @todo Code help screen. Please.
- *
  */
+
 #include "config.h"
 #include <stdlib.h>
 #include <string.h>
@@ -90,8 +88,7 @@ call_flow_create()
     panel = new_panel(newwin(LINES, COLS, 0, 0));
 
     // Initialize Call List specific data
-    info = malloc(sizeof(call_flow_info_t));
-    memset(info, 0, sizeof(call_flow_info_t));
+    info = sng_malloc(sizeof(call_flow_info_t));
 
     // Store it into panel userptr
     set_panel_userptr(panel, (void*) info);
@@ -103,6 +100,13 @@ call_flow_create()
     // Calculate available printable area for messages
     info->flow_win = subwin(win, height - 2 - 2 - 2, width - 2, 4, 0); // Header - Footer - Address
     info->raw_width = 0; // calculated with the available space after drawing columns
+    info->last_msg = NULL;
+
+    // Create vectors for columns and flow arrows
+    info->columns = vector_create(2, 1);
+    vector_set_destroyer(info->columns, vector_generic_destroyer);
+    info->arrows = vector_create(20, 5);
+    vector_set_destroyer(info->arrows, vector_generic_destroyer);
 
     return panel;
 }
@@ -111,28 +115,19 @@ void
 call_flow_destroy(PANEL *panel)
 {
     call_flow_info_t *info;
-    call_flow_column_t *column;
-    call_flow_arrow_t *arrow;
 
     // Free the panel information
     if ((info = call_flow_info(panel))) {
-        // Delete panel columns;
-        while ((column = info->columns)) {
-            info->columns = column->next;
-            free(column);
-        }
-
+        // Delete panel columns
+        vector_destroy(info->columns);
         // Delete panel arrows
-        while ((arrow = info->arrows)) {
-            info->arrows = arrow->next;
-            free(arrow);
-        }
-
+        vector_destroy(info->arrows);
         // Delete panel windows
         delwin(info->flow_win);
         delwin(info->raw_win);
-        free(info);
-
+        // Delete displayed call group
+        call_group_destroy(info->group);
+        sng_free(info);
     }
     // Delete panel window
     delwin(panel_window(panel));
@@ -154,6 +149,7 @@ call_flow_draw(PANEL *panel)
     call_flow_arrow_t *arrow = NULL;
     int height, width, cline = 0;
     char title[256];
+    char callid[256];
 
     // Get panel information
     info = call_flow_info(panel);
@@ -166,7 +162,7 @@ call_flow_draw(PANEL *panel)
     // Set title
     if (call_group_count(info->group) == 1) {
         sprintf(title, "Call flow for %s",
-                call_get_attribute(vector_first(info->group->calls), SIP_ATTR_CALLID));
+                call_get_attribute(vector_first(info->group->calls), SIP_ATTR_CALLID, callid));
     } else {
         sprintf(title, "Call flow for %d dialogs", call_group_count(info->group));
     }
@@ -246,21 +242,28 @@ call_flow_draw_columns(PANEL *panel)
     WINDOW *win;
     sip_msg_t *msg;
     vector_iter_t streams;
+    vector_iter_t columns;
     int flow_height, flow_width;
     const char *coltext;
-    char address[50], *end;
+    char ip_src[80], ip_dst[80];
 
     // Get panel information
     info = call_flow_info(panel);
+
     // Get window of main panel
     win = panel_window(panel);
     getmaxyx(info->flow_win, flow_height, flow_width);
 
     // Load columns
-    for (msg = call_group_get_next_msg(info->group, NULL); msg;
+    for (msg = call_group_get_next_msg(info->group, info->last_msg); msg;
          msg = call_group_get_next_msg(info->group, msg)) {
-        call_flow_column_add(panel, call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID), SRC(msg));
-        call_flow_column_add(panel, call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID), DST(msg));
+        memset(ip_src, 0, sizeof(ip_src));
+        memset(ip_dst, 0, sizeof(ip_dst));
+        msg_get_attribute(msg, SIP_ATTR_SRC, ip_src);
+        msg_get_attribute(msg, SIP_ATTR_DST, ip_dst);
+        call_flow_column_add(panel, msg->call->callid, ip_src);
+        call_flow_column_add(panel, msg->call->callid, ip_dst);
+        info->last_msg = msg;
     }
 
     // Add RTP columns FIXME Really
@@ -277,16 +280,14 @@ call_flow_draw_columns(PANEL *panel)
     }
 
     // Draw vertical columns lines
-    for (column = info->columns; column; column = column->next) {
+    columns = vector_iterator(info->columns);
+    while ((column = vector_iterator_next(&columns))) {
         mvwvline(info->flow_win, 0, 20 + 30 * column->colpos, ACS_VLINE, flow_height);
         mvwhline(win, 3, 10 + 30 * column->colpos, ACS_HLINE, 20);
         mvwaddch(win, 3, 20 + 30 * column->colpos, ACS_TTEE);
 
         // Set bold to this address if it's local
-        strcpy(address, column->addr);
-        if ((end = strchr(address, ':')))
-            *end = '\0';
-        if (is_local_address_str(address) && setting_enabled(SETTING_CF_LOCALHIGHLIGHT))
+        if (is_local_address_str(column->addr) && setting_enabled(SETTING_CF_LOCALHIGHLIGHT))
             wattron(win, A_BOLD);
 
         coltext = sip_address_port_format(column->addr);
@@ -303,18 +304,21 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     call_flow_info_t *info;
     WINDOW *win;
     sdp_media_t *media;
-    const char *msg_time;
     const char *msg_callid;
-    const char *msg_method;
-    const char *msg_src;
-    const char *msg_dst;
+    char msg_method[128];
+    char msg_time[80];
+    char msg_src[80];
+    char msg_dst[80];
+    char sdp_address[80];
+    char sdp_port[80];
     char method[80];
-    char delta[15] = {};
+    char delta[15] = { };
     int height, width;
     char mediastr[40];
     sip_msg_t *msg = arrow->msg;
     vector_iter_t medias;
     int color = 0;
+    int msglen;
 
     // Get panel information
     info = call_flow_info(panel);
@@ -333,15 +337,11 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
         return NULL;
 
     // Get message attributes
-    msg_time = msg_get_attribute(msg, SIP_ATTR_TIME);
-    msg_callid = call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID);
-    msg_method = msg_get_attribute(msg, SIP_ATTR_METHOD);
-    msg_src = msg_get_attribute(msg, SIP_ATTR_SRC);
-    msg_dst = msg_get_attribute(msg, SIP_ATTR_DST);
-
-    // Check method has value. We prefer empty arrows rather than crashes
-    if (!msg_method)
-        msg_method = "";
+    msg_callid = msg->call->callid;
+    msg_get_attribute(msg, SIP_ATTR_METHOD, msg_method);
+    msg_get_attribute(msg, SIP_ATTR_TIME, msg_time);
+    msg_get_attribute(msg, SIP_ATTR_SRC, msg_src);
+    msg_get_attribute(msg, SIP_ATTR_DST, msg_dst);
 
     // Print timestamp
     if (info->selected == arrow)
@@ -349,44 +349,54 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     mvwprintw(win, cline, 2, "%s", msg_time);
 
     // Print delta from selected message
-    if (!info->selected) {
-        if (setting_enabled(SETTING_CF_DELTA))
-            timeval_to_delta(msg_get_time(msg), msg_get_time(call_group_get_next_msg(info->group, msg)), delta);
-    } else if (info->cur_arrow == arrow) {
-        timeval_to_delta(msg_get_time(call_flow_arrow_message(info->selected)), msg_get_time(msg), delta);
-    }
+    if (!setting_has_value(SETTING_CF_SDP_INFO, "compressed")) {
+        if (!info->selected) {
+            if (setting_enabled(SETTING_CF_DELTA))
+                timeval_to_delta(msg_get_time(msg), msg_get_time(call_group_get_next_msg(info->group, msg)), delta);
+        } else if (info->cur_arrow == arrow) {
+            timeval_to_delta(msg_get_time(call_flow_arrow_message(info->selected)), msg_get_time(msg), delta);
+        }
 
-    if (strlen(delta)) {
-        wattron(win, COLOR_PAIR(CP_CYAN_ON_DEF));
-        mvwprintw(win, cline + 1 , 2, "%15s", delta);
+        if (strlen(delta)) {
+            wattron(win, COLOR_PAIR(CP_CYAN_ON_DEF));
+            mvwprintw(win, cline + 1 , 2, "%15s", delta);
+        }
+        wattroff(win, COLOR_PAIR(CP_CYAN_ON_DEF));
     }
-    wattroff(win, COLOR_PAIR(CP_CYAN_ON_DEF));
 
     // Get Message method (include extra info)
     sprintf(method, "%s", msg_method);
 
     // If message has sdp information
     if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "off")) {
-        // Show sdp tag in tittle
+        // Show sdp tag in title
         sprintf(method, "%s (SDP)", msg_method);
     }
 
+    // If message has sdp information
+    if (setting_has_value(SETTING_CF_SDP_INFO, "compressed")) {
+        // Show sdp tag in title
+        if (msg_has_sdp(msg)) {
+            sprintf(method, "%.*s (SDP)", 12, msg_method);
+        } else {
+            sprintf(method, "%.*s", 17, msg_method);
+        }
+    }
+
     if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "first")) {
         sprintf(method, "%.3s (%s:%s)",
                 msg_method,
-                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS),
-                msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
+                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS, sdp_address),
+                msg_get_attribute(msg, SIP_ATTR_SDP_PORT, sdp_port));
     }
     if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
         sprintf(method, "%.3s (%s)",
                 msg_method,
-                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS));
+                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS, sdp_address));
     }
 
     // Draw message type or status and line
-    int msglen = strlen(method);
-    if (msglen > 24)
-        msglen = 24;
+    msglen = (strlen(method) > 24) ? 24 : strlen(method);
 
     // Get origin and destination column
     call_flow_column_t *column1 = call_flow_column_get(panel, msg_callid, msg_src);
@@ -436,7 +446,10 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
     // Draw method
     mvwprintw(win, cline, startpos + distance / 2 - msglen / 2 + 2, "%.26s", method);
-    cline ++;
+
+    if (!setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
+        cline++;
+
     // Draw media information
     if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
         medias = vector_iterator(msg->medias);
@@ -444,7 +457,7 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
             sprintf(mediastr, "%s %d (%s)",
                     media_get_type(media),
                     media_get_port(media),
-                    media_get_format(media));
+                    media_get_prefered_format(media));
             mvwprintw(win, cline++, startpos + distance / 2 - strlen(mediastr) / 2 + 2, mediastr);
         }
     }
@@ -470,6 +483,9 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
         }
     }
 
+    if (setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
+        mvwprintw(win, cline, startpos + distance / 2 - msglen / 2 + 2, " %.26s ", method);
+
     // Turn off colors
     wattroff(win, COLOR_PAIR(CP_RED_ON_DEF));
     wattroff(win, COLOR_PAIR(CP_GREEN_ON_DEF));
@@ -486,9 +502,11 @@ call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
 {
     call_flow_info_t *info;
     WINDOW *win;
-//    char time[20];
-    char codec[50];
+    char codec[50], time[20];
     int height, width;
+    const char *callid;
+    char msg_dst[80], msg_src[80];
+    call_flow_column_t *column1, *column2;
     rtp_stream_t *stream = arrow->stream;
 
     // Get panel information
@@ -508,14 +526,36 @@ call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
         return NULL;
 
     // Print timestamp
-    //mvwprintw(win, cline, 2, "%s", timeval_to_time(stream->time, time));
+    timeval_to_time(stream->time, time);
+    mvwprintw(win, cline, 2, "%s", time);
 
     // Get Message method (include extra info)
-    sprintf(codec, "RTP (%s) %d", rtp_get_codec(stream->format, NULL), stream_get_count(stream));
+    sprintf(codec, "RTP (%s) %d", stream_get_format(stream), stream_get_count(stream));
 
-    // Get origin and destination column
-    call_flow_column_t *column1 = call_flow_column_get(panel, 0, stream->ip_src);
-    call_flow_column_t *column2 = call_flow_column_get(panel, 0, stream->ip_dst);
+    // Get message data
+    msg_get_attribute(stream->media->msg, SIP_ATTR_SRC, msg_src);
+    msg_get_attribute(stream->media->msg, SIP_ATTR_DST, msg_dst);
+    callid = stream->media->msg->call->callid;
+
+    // Get origin column for this stream.
+    // If we share the same Address from its setup SIP packet, use that column instead.
+    if (!strncmp(stream->ip_src, msg_src, strlen(stream->ip_src))) {
+        column1 = call_flow_column_get(panel, callid, msg_src);
+    } else if (!strncmp(stream->ip_src, msg_dst, strlen(stream->ip_src))) {
+        column1 = call_flow_column_get(panel, callid, msg_dst);
+    } else {
+        column1 = call_flow_column_get(panel, 0, stream->ip_src);
+    }
+
+    // Get destination column for this stream.
+    // If we share the same Address from its setup SIP packet, use that column instead.
+    if (!strncmp(stream->ip_dst, msg_dst, strlen(stream->ip_dst))) {
+        column2 = call_flow_column_get(panel, callid, msg_dst);
+    } else if (!strncmp(stream->ip_dst, msg_src, strlen(stream->ip_dst))) {
+        column2 = call_flow_column_get(panel, callid, msg_src);
+    } else {
+        column2 = call_flow_column_get(panel, 0, stream->ip_dst);
+    }
 
     call_flow_column_t *tmp;
     if (column1->colpos > column2->colpos) {
@@ -545,8 +585,8 @@ call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     // Clear the line
     mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
     // Draw method
-    mvwprintw(win, cline, startpos + (distance) / 2 - strlen(codec) / 2 + 2, "%s", codec);
-    cline ++;
+    mvwprintw(win, cline++, startpos + (distance) / 2 - strlen(codec) / 2 + 2, "%s", codec);
+
     // Draw line between columns
     mvwhline(win, cline, startpos + 2, ACS_HLINE, distance);
     // Write the arrow at the end of the message (two arros if this is a retrans)
@@ -588,11 +628,16 @@ call_flow_next_arrow(PANEL *panel, const call_flow_arrow_t *cur)
     rtp_stream_t *stream = NULL;
     struct timeval cur_time;
     call_flow_info_t *info;
-    call_flow_arrow_t *next, *tmp;
+    call_flow_arrow_t *next;
 
     // Get panel information
     info = call_flow_info(panel);
 
+    // Return next arrow if already parsed
+    if (cur && (next = vector_item(info->arrows, cur->index + 1)))
+        return next;
+
+    // Get current arrow timestamp
     if (!cur) {
         memset(&cur_time, 0, sizeof(struct timeval));
     } else if (cur->type == CF_ARROW_SIP) {
@@ -622,63 +667,53 @@ call_flow_next_arrow(PANEL *panel, const call_flow_arrow_t *cur)
 
     /* a rtp stream goes next */
     if (!msg) {
-        if ((next = call_flow_arrow_find(panel, stream)))
-            return next;
         // Create a new arrow to store next info
-        next = malloc(sizeof(call_flow_arrow_t));
-        memset(next, 0, sizeof(call_flow_arrow_t));
+        next = sng_malloc(sizeof(call_flow_arrow_t));
         next->type = CF_ARROW_RTP;
         next->stream = stream;
     } else if (!stream) {
         /* a sip message goes next */
-        if ((next = call_flow_arrow_find(panel, msg)))
-            return next;
         // Create a new arrow to store next info
-        next = malloc(sizeof(call_flow_arrow_t));
-        memset(next, 0, sizeof(call_flow_arrow_t));
+        next = sng_malloc(sizeof(call_flow_arrow_t));
         next->type = CF_ARROW_SIP;
         next->msg = msg;
     } else {
         /* Determine what goes next */
         if (timeval_is_older(msg_get_time(msg), stream->time)) {
-            if ((next = call_flow_arrow_find(panel, stream)))
-                return next;
             // Create a new arrow to store next info
-            next = malloc(sizeof(call_flow_arrow_t));
-            memset(next, 0, sizeof(call_flow_arrow_t));
+            next = sng_malloc(sizeof(call_flow_arrow_t));
             next->type = CF_ARROW_RTP;
             next->stream = stream;
         } else {
-            if ((next = call_flow_arrow_find(panel, msg)))
-                return next;
             // Create a new arrow to store next info
-            next = malloc(sizeof(call_flow_arrow_t));
-            memset(next, 0, sizeof(call_flow_arrow_t));
+            next = sng_malloc(sizeof(call_flow_arrow_t));
             next->type = CF_ARROW_SIP;
             next->msg = msg;
         }
     }
 
     // Add this arrow to the list and return it
-    if (!info->arrows) {
-        info->arrows = next;
-    } else {
-        for (tmp = info->arrows; tmp->next; tmp = tmp->next);
-        tmp->next = next;
-    }
+    next->index = vector_append(info->arrows, next);
+
     return next;
 }
 
 call_flow_arrow_t *
 call_flow_prev_arrow(PANEL *panel, const call_flow_arrow_t *cur)
 {
-    call_flow_arrow_t *arrow = NULL, *prev = NULL;
-    while ((arrow = call_flow_next_arrow(panel, arrow))) {
-        if (arrow == cur) {
-            return prev;
-        }
-        prev = arrow;
-    }
+    call_flow_arrow_t *prev;
+    call_flow_info_t *info;
+
+    if (!cur)
+        return NULL;
+
+    // Get panel information
+    if (!(info = call_flow_info(panel)))
+        return NULL;
+
+    if ((prev = vector_item(info->arrows, cur->index - 1)))
+        return prev;
+
     return NULL;
 }
 
@@ -686,6 +721,8 @@ int
 call_flow_arrow_height(PANEL *panel, const call_flow_arrow_t *arrow)
 {
     if (arrow->type == CF_ARROW_SIP) {
+        if (setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
+            return 1;
         if (!msg_has_sdp(arrow->msg))
             return 2;
         if (setting_has_value(SETTING_CF_SDP_INFO, "off"))
@@ -706,6 +743,7 @@ call_flow_arrow_find(PANEL *panel, const void *data)
 {
     call_flow_info_t *info;
     call_flow_arrow_t *arrow;
+    vector_iter_t arrows;
 
     if (!data)
         return NULL;
@@ -713,7 +751,8 @@ call_flow_arrow_find(PANEL *panel, const void *data)
     if (!(info = call_flow_info(panel)))
         return NULL;
 
-    for (arrow = info->arrows; arrow; arrow = arrow->next)
+    arrows = vector_iterator(info->arrows);
+    while ((arrow = vector_iterator_next(&arrows)))
         if (arrow->msg == data || arrow->stream == data)
             return arrow;
 
@@ -742,7 +781,7 @@ call_flow_draw_raw(PANEL *panel, sip_msg_t *msg)
 
     // Get panel information
     if (!(info = call_flow_info(panel)))
-            return 1;
+        return 1;
 
     // Get window of main panel
     win = panel_window(panel);
@@ -753,7 +792,7 @@ call_flow_draw_raw(PANEL *panel, sip_msg_t *msg)
     fixed_raw_width = setting_get_intvalue(SETTING_CF_RAWFIXEDWIDTH);
 
     // Calculate the raw data width (width - used columns for flow - vertical lines)
-    raw_width = width - (31 + 30 * info->columns->colpos) - 2;
+    raw_width = width - (30 * vector_count(info->columns)) - 2;
     // We can define a mininum size for rawminwidth
     if (raw_width < min_raw_width) {
         raw_width = min_raw_width;
@@ -804,7 +843,7 @@ call_flow_handle_key(PANEL *panel, int key)
     call_flow_info_t *info = call_flow_info(panel);
     call_flow_arrow_t *next, *prev;
     ui_t *next_panel;
-    sip_call_group_t *group;
+    sip_call_t *call;
     int rnpag_steps = setting_get_intvalue(SETTING_CF_SCROLLSTEP);
     int action = -1;
 
@@ -869,12 +908,14 @@ call_flow_handle_key(PANEL *panel, int key)
                 break;
             case ACTION_SHOW_FLOW_EX:
                 werase(panel_window(panel));
-                group = call_group_create();
                 if (call_group_count(info->group) == 1) {
-                    call_group_add(group, call_get_xcall(vector_first(info->group->calls)));
+                    call_group_add(info->group, call_get_xcall(vector_first(info->group->calls)));
+                } else {
+                    call = vector_first(info->group->calls);
+                    vector_clear(info->group->calls);
+                    call_group_add(info->group, call);
                 }
-                call_group_add(group, vector_first(info->group->calls));
-                call_flow_set_group(group);
+                call_flow_set_group(info->group);
                 break;
             case ACTION_SHOW_RAW:
                 // KEY_R, display current call in raw mode
@@ -910,6 +951,8 @@ call_flow_handle_key(PANEL *panel, int key)
                 break;
             case ACTION_TOGGLE_MEDIA:
                 setting_toggle(SETTING_CF_MEDIA);
+                // Force reload arrows
+                call_flow_set_group(info->group);
                 break;
             case ACTION_TOGGLE_RAW:
                 setting_toggle(SETTING_CF_FORCERAW);
@@ -1026,11 +1069,14 @@ call_flow_set_group(sip_call_group_t *group)
     if (!(info = call_flow_info(panel)))
         return -1;
 
+    vector_clear(info->columns);
+    vector_clear(info->arrows);
+
     info->group = group;
-    info->cur_arrow = info->first_arrow = call_flow_next_arrow(panel, NULL);
+    info->cur_arrow = info->first_arrow = info->selected = call_flow_next_arrow(panel, NULL);
     info->cur_line = 1;
-    info->columns = NULL;
     info->selected = NULL;
+    info->last_msg = NULL;
 
     return 0;
 }
@@ -1040,7 +1086,7 @@ call_flow_column_add(PANEL *panel, const char *callid, const char *addr)
 {
     call_flow_info_t *info;
     call_flow_column_t *column;
-    int colpos = 0;
+    vector_iter_t columns;
 
     if (!(info = call_flow_info(panel)))
         return;
@@ -1051,32 +1097,27 @@ call_flow_column_add(PANEL *panel, const char *callid, const char *addr)
     if (call_flow_column_get(panel, callid, addr))
         return;
 
-    column = info->columns;
-    while (column) {
+    columns = vector_iterator(info->columns);
+    while ((column = vector_iterator_next(&columns))) {
         if (!strcasecmp(addr, column->addr) && column->colpos != 0 && !column->callid2) {
             column->callid2 = callid;
             return;
         }
-        column = column->next;
     }
 
-    if (info->columns)
-        colpos = info->columns->colpos + 1;
-
-    column = malloc(sizeof(call_flow_column_t));
-    memset(column, 0, sizeof(call_flow_column_t));
+    column = sng_malloc(sizeof(call_flow_column_t));
     column->callid = callid;
-    column->addr = addr;
-    column->colpos = colpos;
-    column->next = info->columns;
-    info->columns = column;
+    strcpy(column->addr, addr);
+    column->colpos = vector_count(info->columns);
+    vector_append(info->columns, column);
 }
 
 call_flow_column_t *
 call_flow_column_get(PANEL *panel, const char *callid, const char *addr)
 {
     call_flow_info_t *info;
-    call_flow_column_t *columns;
+    call_flow_column_t *column;
+    vector_iter_t columns;
     int match_port;
     char coladdr[ADDRESSLEN + 6];
     char *dots;
@@ -1087,10 +1128,10 @@ call_flow_column_get(PANEL *panel, const char *callid, const char *addr)
     // Look for address or address:port ?
     match_port = (strchr(addr, ':') != NULL);
 
-    columns = info->columns;
-    while (columns) {
+    columns = vector_iterator(info->columns);
+    while ((column = vector_iterator_next(&columns))) {
         // Copy address:port column label
-        strcpy(coladdr, columns->addr);
+        strcpy(coladdr, column->addr);
 
         // Remove port if we want to match only address
         if (!match_port && (dots = strchr(coladdr, ':')))
@@ -1098,15 +1139,14 @@ call_flow_column_get(PANEL *panel, const char *callid, const char *addr)
 
         if (!strcasecmp(addr, coladdr)) {
             if (!match_port)
-                return columns;
+                return column;
             if (setting_enabled(SETTING_CF_SPLITCALLID))
-                return columns;
-            if (columns->callid && !strcasecmp(callid, columns->callid))
-                return columns;
-            if (columns->callid2 && !strcasecmp(callid, columns->callid2))
-                return columns;
+                return column;
+            if (column->callid && !strcasecmp(callid, column->callid))
+                return column;
+            if (column->callid2 && !strcasecmp(callid, column->callid2))
+                return column;
         }
-        columns = columns->next;
     }
     return NULL;
 }
