@@ -69,8 +69,7 @@ call_list_create()
     // Create a new panel that fill all the screen
     panel = new_panel(newwin(LINES, COLS, 0, 0));
     // Initialize Call List specific data
-    info = malloc(sizeof(call_list_info_t));
-    memset(info, 0, sizeof(call_list_info_t));
+    info = sng_malloc(sizeof(call_list_info_t));
     // Store it into panel userptr
     set_panel_userptr(panel, (void*) info);
 
@@ -116,12 +115,6 @@ call_list_create()
     vector_iterator_set_filter(&info->calls, filter_check_call);
     info->cur_call = info->first_call = -1;
 
-    // Set defualt filter text if configured
-    if (setting_get_value(SETTING_CL_FILTER)) {
-        set_field_buffer(info->fields[FLD_LIST_FILTER], 0, setting_get_value(SETTING_CL_FILTER));
-        call_list_form_activate(panel, 0);
-    }
-
     // Return the created panel
     return panel;
 }
@@ -141,12 +134,11 @@ call_list_destroy(PANEL *panel)
         }
 
         // Deallocate group data
-        if (info->group)
-            call_group_destroy(info->group);
+        call_group_destroy(info->group);
 
         // Deallocate panel windows
         delwin(info->list_win);
-        free(info);
+        sng_free(info);
     }
 
     // Deallocate panel window
@@ -260,11 +252,14 @@ call_list_draw_footer(PANEL *panel)
 void
 call_list_draw_list(PANEL *panel)
 {
-
+    WINDOW *win;
     int height, width, cline = 0;
     struct sip_call *call;
-    char linetext[256];
-    WINDOW *win;
+    int i, collen;
+    char coltext[256];
+    int colid;
+    int colpos;
+    int color;
 
     // Get panel info
     call_list_info_t *info = call_list_info(panel);
@@ -297,26 +292,55 @@ call_list_draw_list(PANEL *panel)
         if (!call_msg_count(call))
             continue;
 
+        // Reverse colors on monochrome terminals
+        if (!has_colors())
+            wattron(win, A_REVERSE);
+
         // Show bold selected rows
         if (call_group_exists(info->group, call))
             wattron(win, A_BOLD | COLOR_PAIR(CP_DEFAULT));
 
         // Highlight active call
-        if (call == vector_item(vector_iterator_vector(&info->calls), info->cur_call)) {
-            // Reverse colors on monochrome terminals
-            if (!has_colors())
-                wattron(win, A_REVERSE);
+        if (call->index == info->cur_call + 1)
             wattron(win, COLOR_PAIR(CP_DEF_ON_BLUE));
-        }
-
         // Set current line background
         clear_line(win, cline);
         // Set current line selection box
         mvwprintw(win, cline, 2, call_group_exists(info->group, call) ? "[*]" : "[ ]");
 
-        // Print call line
-        memset(linetext, 0, sizeof(linetext));
-        mvwprintw(win, cline, 6, "%-*s", width - 6, call_list_line_text(panel, call, linetext));
+        // Print requested columns
+        colpos = 6;
+        for (i = 0; i < info->columncnt; i++) {
+            // Get current column id
+            colid = info->columns[i].id;
+            // Get current column width
+            collen = info->columns[i].width;
+            // Check if next column fits on window width
+            if (colpos + collen >= width)
+                break;
+
+            // Initialize column text
+            memset(coltext, 0, sizeof(coltext));
+
+            // Get call attribute for current column
+            if (!call_get_attribute(call, colid, coltext)) {
+                colpos += collen + 1;
+                continue;
+            }
+
+            // Enable attribute color (if not current one)
+            color = 0;
+            if (call->index != info->cur_call + 1 && (color = sip_attr_get_color(colid, coltext)) > 0)
+                wattron(win, color);
+
+            // Add the column text to the existing columns
+            mvwprintw(win, cline, colpos, "%.*s", collen, coltext);
+            colpos += collen + 1;
+
+            // Disable attribute color
+            if (color > 0)
+                wattroff(win, color);
+        }
         cline++;
 
         wattroff(win, COLOR_PAIR(CP_DEFAULT));
@@ -380,7 +404,7 @@ const char *
 call_list_line_text(PANEL *panel, sip_call_t *call, char *text)
 {
     int i, collen;
-    const char *call_attr;
+    char call_attr[256];
     char coltext[256];
     int colid;
     int width;
@@ -412,8 +436,10 @@ call_list_line_text(PANEL *panel, sip_call_t *call, char *text)
 
         // Initialize column text
         memset(coltext, 0, sizeof(coltext));
+        memset(call_attr, 0, sizeof(call_attr));
+
         // Get call attribute for current column
-        if ((call_attr = call_get_attribute(call, colid))) {
+        if (call_get_attribute(call, colid, call_attr)) {
             sprintf(coltext, "%.*s", collen, call_attr);
         }
         // Add the column text to the existing columns
@@ -498,53 +524,57 @@ call_list_handle_key(PANEL *panel, int key)
                 for (i = 0; i < rnpag_steps; i++)
                     call_list_handle_key(panel, KEY_UP);
                 break;
+            case ACTION_BEGIN:
+                // Initialize structures
+                info->first_call = info->cur_call = -1;
+                info->first_line = info->cur_line = 0;
+                break;
+            case ACTION_END:
+                // Check if there is a call below us
+                while (vector_iterator_next(&info->calls)) {
+                    info->cur_call = vector_iterator_current(&info->calls);
+                    info->cur_line++;
+                    // If we are out of the bottom of the displayed list
+                    // refresh it starting in the next call
+                    if (info->cur_line > height) {
+                        vector_iterator_set_current(&info->calls, info->first_call);
+                        vector_iterator_next(&info->calls);
+                        info->first_call = vector_iterator_current(&info->calls);
+                        info->first_line++;
+                        info->cur_line = height;
+                        vector_iterator_set_current(&info->calls, info->cur_call);
+                    }
+                }
+                break;
             case ACTION_DISP_FILTER:
                 // Activate Form
                 call_list_form_activate(panel, 1);
                 break;
             case ACTION_SHOW_FLOW:
-                // Check we have calls in the list
-                if (info->cur_call == -1)
-                    break;
-                ui_create_panel(PANEL_CALL_FLOW);
-                if (call_group_count(info->group)) {
-                    group = info->group;
-                } else {
-                    group = call_group_create();
-                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
-                    call_group_add(group, call);
-                }
-                call_flow_set_group(group);
-                break;
             case ACTION_SHOW_FLOW_EX:
-                // Check we have calls in the list
-                if (info->cur_call == -1)
-                    break;
-                // Display current call flow (extended)
-                ui_create_panel(PANEL_CALL_FLOW);
-                if (call_group_count(info->group)) {
-                    group = info->group;
-                } else {
-                    group = call_group_create();
-                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
-                    call_group_add(group, call);
-                    call_group_add(group, call_get_xcall(call));
-                }
-                call_flow_set_group(group);
-                break;
             case ACTION_SHOW_RAW:
                 // Check we have calls in the list
                 if (info->cur_call == -1)
                     break;
-                ui_create_panel(PANEL_CALL_RAW);
-                if (call_group_count(info->group)) {
-                    group = info->group;
+                // Create a new group of calls
+                group = call_group_clone(info->group);
+                // If not selected call, show current call flow
+                if (call_group_count(info->group) == 0)
+                    call_group_add(group, sip_find_by_index(info->cur_call));
+
+                // Add xcall to the group
+                if (action == ACTION_SHOW_FLOW_EX)
+                    call_group_add(group, call_get_xcall(sip_find_by_index(info->cur_call)));
+
+                if (action == ACTION_SHOW_RAW) {
+                    // Create a Call Flow panel
+                    ui_create_panel(PANEL_CALL_RAW);
+                    call_raw_set_group(group);
                 } else {
-                    group = call_group_create();
-                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
-                    call_group_add(group, call);
+                    // Display current call flow (normal or extended)
+                    ui_create_panel(PANEL_CALL_FLOW);
+                    call_flow_set_group(group);
                 }
-                call_raw_set_group(group);
                 break;
             case ACTION_SHOW_FILTERS:
                 ui_create_panel(PANEL_FILTER);
@@ -563,6 +593,10 @@ call_list_handle_key(PANEL *panel, int key)
                 call_list_clear(panel);
                 filter_reset_calls();
                 break;
+            case ACTION_CLEAR:
+                // Clear group calls
+                vector_clear(info->group->calls);
+                break;
             case ACTION_CLEAR_CALLS:
                 // Remove all stored calls
                 sip_calls_clear();
@@ -583,7 +617,11 @@ call_list_handle_key(PANEL *panel, int key)
             case ACTION_PREV_SCREEN:
                 // Handle quit from this screen unless requested
                 if (setting_enabled(SETTING_EXITPROMPT)) {
-                    return call_list_exit_confirm(panel);
+                    if (dialog_confirm("Confirm exit", "Are you sure you want to quit?", "Yes,No") == 0) {
+                        return KEY_ESC;
+                    } else {
+                        return 0;
+                    }
                 } else {
                     return KEY_ESC;
                 }
@@ -621,10 +659,6 @@ call_list_handle_form_key(PANEL *panel, int key)
             case ACTION_PRINTABLE:
                 // If this is a normal character on input field, print it
                 form_driver(info->form, key);
-                // Updated displayed results
-                call_list_clear(panel);
-                // Reset filters on each key stroke
-                filter_reset_calls();
                 break;
             case ACTION_PREV_SCREEN:
             case ACTION_NEXT_FIELD:
@@ -656,10 +690,6 @@ call_list_handle_form_key(PANEL *panel, int key)
                 break;
             case ACTION_BACKSPACE:
                 form_driver(info->form, REQ_DEL_PREV);
-                // Updated displayed results
-                call_list_clear(panel);
-                // Reset filters on each key stroke
-                filter_reset_calls();
                 break;
             default:
                 // Parse next action
@@ -670,13 +700,23 @@ call_list_handle_form_key(PANEL *panel, int key)
         break;
     }
 
+    // Filter has changed, re-apply filter to displayed calls
+    if (action == ACTION_PRINTABLE || action == ACTION_BACKSPACE ||
+            action == ACTION_DELETE || action == ACTION_CLEAR) {
+        // Updated displayed results
+         call_list_clear(panel);
+         // Reset filters on each key stroke
+         filter_reset_calls();
+    }
+
     // Validate all input data
     form_driver(info->form, REQ_VALIDATION);
 
     // Store dfilter input
     // We trim spaces with sscanf because and empty field is stored as space characters
     memset(dfilter, 0, sizeof(dfilter));
-    sscanf(field_buffer(info->fields[FLD_LIST_FILTER], 0), "%[^ ]", dfilter);
+    strcpy(dfilter, field_buffer(info->fields[FLD_LIST_FILTER], 0));
+    strtrim(dfilter);
 
     // Set display filter
     filter_set(FILTER_CALL_LIST, strlen(dfilter) ? dfilter : NULL);
@@ -748,72 +788,6 @@ call_list_help(PANEL *panel)
     return 0;
 }
 
-int
-call_list_exit_confirm(PANEL *panel)
-{
-    WINDOW *exit_win;
-    int c;
-
-    // Initial exit status
-    int exit = 1;
-
-    // Create a new panel and show centered
-    exit_win = newwin(8, 40, (LINES - 8) / 2, (COLS - 40) / 2);
-    keypad(exit_win, TRUE);
-
-    // Set the window title
-    mvwprintw(exit_win, 1, 13, "Confirm exit");
-
-    // Write border and boxes around the window
-    wattron(exit_win, COLOR_PAIR(CP_BLUE_ON_DEF));
-    box(exit_win, 0, 0);
-    mvwhline(exit_win, 2, 1, ACS_HLINE, 40);
-    mvwhline(exit_win, 5, 1, ACS_HLINE, 40);
-    mvwaddch(exit_win, 2, 0, ACS_LTEE);
-    mvwaddch(exit_win, 5, 0, ACS_LTEE);
-    mvwaddch(exit_win, 2, 39, ACS_RTEE);
-    mvwaddch(exit_win, 5, 39, ACS_RTEE);
-
-    // Exit confirmation message message
-    wattron(exit_win, COLOR_PAIR(CP_CYAN_ON_DEF));
-    mvwprintw(exit_win, 3, 2, "Are you sure you want to quit?");
-    wattroff(exit_win, COLOR_PAIR(CP_CYAN_ON_DEF));
-
-    for (;;) {
-        // A list of available keys in this window
-        if (exit)
-            wattron(exit_win, A_REVERSE);
-        mvwprintw(exit_win, 6, 10, "[  Yes  ]");
-        wattroff(exit_win, A_REVERSE);
-        if (!exit)
-            wattron(exit_win, A_REVERSE);
-        mvwprintw(exit_win, 6, 20, "[  No   ]");
-        wattroff(exit_win, A_REVERSE);
-
-        c = wgetch(exit_win);
-        switch (c) {
-            case KEY_RIGHT:
-                exit = 0;
-                break;
-            case KEY_LEFT:
-                exit = 1;
-                break;
-            case KEY_TAB:
-                exit = (exit) ? 0 : 1;
-                break;
-            case KEY_SPACE:
-            case KEY_ENTER:
-            case KEY_INTRO:
-                delwin(exit_win);
-                // If we return ESC, we let ui_manager to handle this
-                // key and exit sngrep gracefully
-                return (exit) ? KEY_ESC : 0;
-        }
-    }
-
-    return 0;
-}
-
 int
 call_list_add_column(PANEL *panel, enum sip_attr_id id, const char* attr, const char *title,
                      int width)
