@@ -44,44 +44,48 @@ ui_t ui_settings = {
     .destroy = settings_destroy
 };
 
-settings_category_t categories[] =
-{
- { CAT_SETTINGS_INTERFACE,  "Interface" },
- { CAT_SETTINGS_CAPTURE,    "Capture" },
- { CAT_SETTINGS_CALL_FLOW,  "Call Flow" },
- { CAT_SETTINGS_CALL_LIST,  "Call List" },
- { CAT_SETTINGS_CALL_RAW,  "Call Raw" },
- { 0 , NULL },
+settings_category_t categories[] = {
+    { CAT_SETTINGS_INTERFACE,  "Interface" },
+    { CAT_SETTINGS_CAPTURE,    "Capture" },
+    { CAT_SETTINGS_CALL_FLOW,  "Call Flow" },
+    { CAT_SETTINGS_EEP_HOMER,  "EEP/HEP Homer" },
+    { 0 , NULL },
 };
 
 settings_entry_t entries[] = {
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_BACKGROUND,         SETTING_BACKGROUND,         "Background * .............................." },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX,             SETTING_SYNTAX,             "SIP message syntax ........................" },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX_TAG,         SETTING_SYNTAX_TAG,         "SIP tag syntax ............................" },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX_BRANCH,      SETTING_SYNTAX_BRANCH,      "SIP branch syntax ........................." },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_ALTKEY_HINT,        SETTING_ALTKEY_HINT,        "Alternative keybinding hints .............." },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_COLORMODE,          SETTING_COLORMODE,          "Default message color mode ................" },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_EXITPROMPT,         SETTING_EXITPROMPT,         "Always prompt on quit ....................." },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_DISPLAY_HOST,       SETTING_DISPLAY_HOST,       "Replace addresses with resolved hosts ....." },
- { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_DISPLAY_ALIAS,      SETTING_DISPLAY_ALIAS,      "Replace addresses with alias .............." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_RTP,        SETTING_CAPTURE_RTP,        "Capture RTP packets * ....................." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_LIMIT,      SETTING_CAPTURE_LIMIT,      "Max dialogs * ............................." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_DEVICE,     SETTING_CAPTURE_DEVICE,     "Capture device * .........................." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_LOOKUP,     SETTING_CAPTURE_LOOKUP,     "Resolved IP Addresses * ..................." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SIP_NOINCOMPLETE,   SETTING_SIP_NOINCOMPLETE,   "Capture full transactions ................." },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SIP_CALLS,          SETTING_SIP_CALLS,          "Only capture calls * ......................" },
- { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SAVEPATH,           SETTING_SAVEPATH,           "Default Save path ........................." },
- { CAT_SETTINGS_CALL_LIST,  FLD_SETTINGS_CL_SCROLLSTEP,      SETTING_CL_SCROLLSTEP,      "Steps for PgUp/PgDown ....................." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_FORCERAW,        SETTING_CF_FORCERAW,        "Show message preview panel ................" },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_HIGHTLIGHT,      SETTING_CF_HIGHTLIGHT,      "Selected message hightlight ..............." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_LOCALHIGHLIGHT,  SETTING_CF_LOCALHIGHLIGHT,  "Highlight local addresses ................." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SPLITCACALLID,   SETTING_CF_SPLITCALLID,     "Merge columns with same address ..........." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SDPONLY,         SETTING_CF_SDP_INFO,        "Show SDP information in messages .........." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_DELTA,           SETTING_CF_DELTA,           "Show delta time between messages .........." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_MEDIA,           SETTING_CF_MEDIA,           "Show RTP media streams ...................." },
- { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SCROLLSTEP,      SETTING_CF_SCROLLSTEP,      "Steps for PgUp/PgDown ....................." },
- { CAT_SETTINGS_CALL_RAW,   FLD_SETTINGS_CR_SCROLLSTEP,      SETTING_CR_SCROLLSTEP,      "Steps for PgUp/PgDown ....................." },
- { 0 , 0, 0, NULL },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_BACKGROUND,         SETTING_BACKGROUND,         "Background * .............................." },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX,             SETTING_SYNTAX,             "SIP message syntax ........................" },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX_TAG,         SETTING_SYNTAX_TAG,         "SIP tag syntax ............................" },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_SYNTAX_BRANCH,      SETTING_SYNTAX_BRANCH,      "SIP branch syntax ........................." },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_ALTKEY_HINT,        SETTING_ALTKEY_HINT,        "Alternative keybinding hints .............." },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_COLORMODE,          SETTING_COLORMODE,          "Default message color mode ................" },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_EXITPROMPT,         SETTING_EXITPROMPT,         "Always prompt on quit ....................." },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_DISPLAY_HOST,       SETTING_DISPLAY_HOST,       "Replace addresses with resolved hosts ....." },
+    { CAT_SETTINGS_INTERFACE,  FLD_SETTINGS_DISPLAY_ALIAS,      SETTING_DISPLAY_ALIAS,      "Replace addresses with alias .............." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_RTP,        SETTING_CAPTURE_RTP,        "Capture RTP packets * ....................." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_LIMIT,      SETTING_CAPTURE_LIMIT,      "Max dialogs * ............................." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_DEVICE,     SETTING_CAPTURE_DEVICE,     "Capture device * .........................." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_CAPTURE_LOOKUP,     SETTING_CAPTURE_LOOKUP,     "Resolved IP Addresses * ..................." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SIP_NOINCOMPLETE,   SETTING_SIP_NOINCOMPLETE,   "Capture full transactions ................." },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SIP_CALLS,          SETTING_SIP_CALLS,          "Only capture calls * ......................" },
+    { CAT_SETTINGS_CAPTURE,    FLD_SETTINGS_SAVEPATH,           SETTING_SAVEPATH,           "Default Save path ........................." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_FORCERAW,        SETTING_CF_FORCERAW,        "Show message preview panel ................" },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_HIGHTLIGHT,      SETTING_CF_HIGHTLIGHT,      "Selected message hightlight ..............." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_LOCALHIGHLIGHT,  SETTING_CF_LOCALHIGHLIGHT,  "Highlight local addresses ................." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SPLITCACALLID,   SETTING_CF_SPLITCALLID,     "Merge columns with same address ..........." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SDPONLY,         SETTING_CF_SDP_INFO,        "Show SDP information in messages .........." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_DELTA,           SETTING_CF_DELTA,           "Show delta time between messages .........." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_MEDIA,           SETTING_CF_MEDIA,           "Show RTP media streams ...................." },
+    { CAT_SETTINGS_CALL_FLOW,  FLD_SETTINGS_CF_SCROLLSTEP,      SETTING_CF_SCROLLSTEP,      "Steps for PgUp/PgDown ....................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_SEND,           SETTING_EEP_SEND,           "Send all captured SIP packets ............." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_SEND_ADDR,      SETTING_EEP_SEND_ADDR,      "Send EEP packet address ..................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_SEND_PORT,      SETTING_EEP_SEND_PORT,      "Send EEP packet port ......................" },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_SEND_PASS,      SETTING_EEP_SEND_PASS,      "EEP send password ........................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_LISTEN,         SETTING_EEP_LISTEN,         "Listen for eep packets ...................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_LISTEN_ADDR,    SETTING_EEP_LISTEN_ADDR,    "Listen EEP packet address ................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_LISTEN_PORT,    SETTING_EEP_LISTEN_PORT,    "Listen EEP packet port ...................." },
+    { CAT_SETTINGS_EEP_HOMER,  FLD_SETTINGS_EEP_LISTEN_PASS,    SETTING_EEP_LISTEN_PASS,    "EEP server password ......................." },
+    { 0 , 0, 0, NULL },
 };
 
 PANEL *
@@ -105,8 +109,7 @@ settings_create()
     panel = new_panel(win);
 
     // Initialize Filter panel specific data
-    info = malloc(sizeof(settings_info_t));
-    memset(info, 0, sizeof(settings_info_t));
+    info = sng_malloc(sizeof(settings_info_t));
 
     // Store it into panel userptr
     set_panel_userptr(panel, (void*) info);
@@ -188,7 +191,7 @@ settings_create()
     // Set the window title and boxes
     mvwprintw(win, 1, width / 2 - 5, "Settings");
     wattron(win, COLOR_PAIR(CP_BLUE_ON_DEF));
-    title_foot_box(panel_window(panel));
+    title_foot_box(panel);
     mvwhline(win, 6, 1, ACS_HLINE, width - 1);
     mvwaddch(win, 6, 0, ACS_LTEE);
     mvwaddch(win, 6, width - 1, ACS_RTEE);
@@ -299,129 +302,129 @@ settings_handle_key(PANEL *panel, int key)
     }
 
     // Check actions for this key
-     while ((action = key_find_action(key, action)) != ERR) {
-         if (info->active_form == info->form) {
-             // Check if we handle this action
-             switch (action) {
-                 case ACTION_PRINTABLE:
-                     if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
-                         form_driver(info->form, key);
-                         break;
-                     }
-                     continue;
-                 case ACTION_UP:
-                 case ACTION_HPPAGE:
-                     form_driver(info->form, REQ_PREV_FIELD);
-                     form_driver(info->form, REQ_END_LINE);
-                     break;
-                 case ACTION_DOWN:
-                 case ACTION_HNPAGE:
-                     form_driver(info->form, REQ_NEXT_FIELD);
-                     form_driver(info->form, REQ_END_LINE);
-                     break;
-                 case ACTION_SELECT:
-                 case ACTION_RIGHT:
-                     form_driver(info->form, REQ_NEXT_CHOICE);
-                     form_driver(info->form, REQ_RIGHT_CHAR);
-                     break;
-                 case ACTION_LEFT:
-                     form_driver(info->form, REQ_PREV_CHOICE);
-                     form_driver(info->form, REQ_LEFT_CHAR);
-                     break;
-                 case ACTION_NPAGE:
-                     form_driver(info->form, REQ_NEXT_PAGE);
-                     form_driver(info->form, REQ_END_LINE);
-                     info->active_category = form_page(info->form) + 1;
-                     break;
-                 case ACTION_PPAGE:
-                     form_driver(info->form, REQ_PREV_PAGE);
-                     form_driver(info->form, REQ_END_LINE);
-                     info->active_category = form_page(info->form) + 1;
-                     break;
-                 case ACTION_BEGIN:
-                     form_driver(info->form, REQ_BEG_LINE);
-                     break;
-                 case ACTION_END:
-                     form_driver(info->form, REQ_END_LINE);
-                     break;
-                 case ACTION_NEXT_FIELD:
-                     info->active_form = info->buttons_form;
-                     set_current_field(info->active_form, info->buttons[BTN_SETTINGS_ACCEPT]);
-                     break;
-                 case ACTION_CLEAR:
-                     if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
-                         form_driver(info->form, REQ_BEG_LINE);
-                         form_driver(info->form, REQ_CLR_EOL);
-                     }
-                     break;
-                 case ACTION_DELETE:
-                     if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
-                         form_driver(info->form, REQ_DEL_CHAR);
-                     }
-                     break;
-                 case ACTION_BACKSPACE:
-                     if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
-                         form_driver(info->form, REQ_DEL_PREV);
-                     }
-                     break;
-                 case ACTION_CONFIRM:
-                     ui_settings_update_settings(panel);
-                     return KEY_ESC;
-                 default:
-                     // Parse next action
-                     continue;
-             }
-         } else {
-             // Check if we handle this action
-             switch (action) {
-                 case ACTION_RIGHT:
-                 case ACTION_DOWN:
-                 case ACTION_NEXT_FIELD:
-                     if (field_idx == BTN_SETTINGS_CANCEL) {
-                         info->active_form = info->form;
-                     } else {
-                         form_driver(info->buttons_form, REQ_NEXT_FIELD);
-                     }
-                     break;
-                 case ACTION_LEFT:
-                 case ACTION_UP:
-                 case ACTION_PREV_FIELD:
-                     if (field_idx == BTN_SETTINGS_ACCEPT) {
-                         info->active_form = info->form;
-                     } else {
-                         form_driver(info->buttons_form, REQ_PREV_FIELD);
-                     }
-                     break;
-                 case ACTION_SELECT:
-                 case ACTION_CONFIRM:
-                     if (field_idx == BTN_SETTINGS_CANCEL)
-                         return KEY_ESC;
-                     if (field_idx == BTN_SETTINGS_SAVE)
-                         ui_settings_save(panel);
-                     ui_settings_update_settings(panel);
-                     return KEY_ESC;
-                 default:
-                     continue;
-             }
-         }
+    while ((action = key_find_action(key, action)) != ERR) {
+        if (info->active_form == info->form) {
+            // Check if we handle this action
+            switch (action) {
+                case ACTION_PRINTABLE:
+                    if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
+                        form_driver(info->form, key);
+                        break;
+                    }
+                    continue;
+                case ACTION_UP:
+                case ACTION_HPPAGE:
+                    form_driver(info->form, REQ_PREV_FIELD);
+                    form_driver(info->form, REQ_END_LINE);
+                    break;
+                case ACTION_DOWN:
+                case ACTION_HNPAGE:
+                    form_driver(info->form, REQ_NEXT_FIELD);
+                    form_driver(info->form, REQ_END_LINE);
+                    break;
+                case ACTION_SELECT:
+                case ACTION_RIGHT:
+                    form_driver(info->form, REQ_NEXT_CHOICE);
+                    form_driver(info->form, REQ_RIGHT_CHAR);
+                    break;
+                case ACTION_LEFT:
+                    form_driver(info->form, REQ_PREV_CHOICE);
+                    form_driver(info->form, REQ_LEFT_CHAR);
+                    break;
+                case ACTION_NPAGE:
+                    form_driver(info->form, REQ_NEXT_PAGE);
+                    form_driver(info->form, REQ_END_LINE);
+                    info->active_category = form_page(info->form) + 1;
+                    break;
+                case ACTION_PPAGE:
+                    form_driver(info->form, REQ_PREV_PAGE);
+                    form_driver(info->form, REQ_END_LINE);
+                    info->active_category = form_page(info->form) + 1;
+                    break;
+                case ACTION_BEGIN:
+                    form_driver(info->form, REQ_BEG_LINE);
+                    break;
+                case ACTION_END:
+                    form_driver(info->form, REQ_END_LINE);
+                    break;
+                case ACTION_NEXT_FIELD:
+                    info->active_form = info->buttons_form;
+                    set_current_field(info->active_form, info->buttons[BTN_SETTINGS_ACCEPT]);
+                    break;
+                case ACTION_CLEAR:
+                    if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
+                        form_driver(info->form, REQ_BEG_LINE);
+                        form_driver(info->form, REQ_CLR_EOL);
+                    }
+                    break;
+                case ACTION_DELETE:
+                    if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
+                        form_driver(info->form, REQ_DEL_CHAR);
+                    }
+                    break;
+                case ACTION_BACKSPACE:
+                    if (sett_fmt == SETTING_FMT_NUMBER || sett_fmt == SETTING_FMT_STRING) {
+                        form_driver(info->form, REQ_DEL_PREV);
+                    }
+                    break;
+                case ACTION_CONFIRM:
+                    ui_settings_update_settings(panel);
+                    return KEY_ESC;
+                default:
+                    // Parse next action
+                    continue;
+            }
+        } else {
+            // Check if we handle this action
+            switch (action) {
+                case ACTION_RIGHT:
+                case ACTION_DOWN:
+                case ACTION_NEXT_FIELD:
+                    if (field_idx == BTN_SETTINGS_CANCEL) {
+                        info->active_form = info->form;
+                    } else {
+                        form_driver(info->buttons_form, REQ_NEXT_FIELD);
+                    }
+                    break;
+                case ACTION_LEFT:
+                case ACTION_UP:
+                case ACTION_PREV_FIELD:
+                    if (field_idx == BTN_SETTINGS_ACCEPT) {
+                        info->active_form = info->form;
+                    } else {
+                        form_driver(info->buttons_form, REQ_PREV_FIELD);
+                    }
+                    break;
+                case ACTION_SELECT:
+                case ACTION_CONFIRM:
+                    if (field_idx == BTN_SETTINGS_CANCEL)
+                        return KEY_ESC;
+                    if (field_idx == BTN_SETTINGS_SAVE)
+                        ui_settings_save(panel);
+                    ui_settings_update_settings(panel);
+                    return KEY_ESC;
+                default:
+                    continue;
+            }
+        }
 
-         // This panel has handled the key successfully
-         break;
-     }
+        // This panel has handled the key successfully
+        break;
+    }
 
-     // Validate all input data
-     form_driver(info->active_form, REQ_VALIDATION);
+    // Validate all input data
+    form_driver(info->active_form, REQ_VALIDATION);
 
-     // Get current setting id
-     if ((entry = ui_settings_is_entry(current_field(info->active_form)))) {
-         // Enable cursor on string and number fields
-         curs_set(setting_format(entry->setting_id) != SETTING_FMT_ENUM);
-     } else {
-         curs_set(0);
-     }
+    // Get current setting id
+    if ((entry = ui_settings_is_entry(current_field(info->active_form)))) {
+        // Enable cursor on string and number fields
+        curs_set(setting_format(entry->setting_id) != SETTING_FMT_ENUM);
+    } else {
+        curs_set(0);
+    }
 
-     // Return if this panel has handled or not the key
-     return (action == ERR) ? key : 0;
+    // Return if this panel has handled or not the key
+    return (action == ERR) ? key : 0;
 }
 
 settings_entry_t *
@@ -484,7 +487,7 @@ ui_settings_save(PANEL *panel)
     rename(userconf, tmpfile);
 
     // Create a new user conf file
-    if (!(fo = fopen(userconf, "w")))  {
+    if (!(fo = fopen(userconf, "w"))) {
         return;
     }
 
