@@ -30,15 +30,15 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include "util.h"
 
 vector_t *
 vector_create(int limit, int step)
 {
     vector_t *v;
     // Allocate memory for this vector data
-    if (!(v = malloc(sizeof(vector_t))))
+    if (!(v = sng_malloc(sizeof(vector_t))))
         return NULL;
-    memset(v, 0, sizeof(vector_t));
     v->limit = limit;
     v->step = step;
     return v;
@@ -48,15 +48,39 @@ void
 vector_destroy(vector_t *vector)
 {
     // Nothing to free. Done.
-    if (!vector)
-        return;
+    if (!vector) return;
     // Remove all items if a destroyer is set
     vector_clear(vector);
     // Deallocate vector list
-    if (vector->list)
-        free(vector->list);
+    sng_free(vector->list);
     // Deallocate vector itself
-    free(vector);
+    sng_free(vector);
+}
+
+
+vector_t *
+vector_clone(vector_t *original)
+{
+    vector_t *clone;
+    vector_iter_t it;
+    void *item;
+
+    // Check we have a valid vector pointer
+    if (!original)
+        return NULL;
+
+    // Create a new vector structure
+    clone = vector_create(original->limit, original->step);
+    vector_set_destroyer(clone, original->destroyer);
+    vector_set_sorter(clone, original->sorter);
+
+    // Fill the clone vector with the same elements
+    it = vector_iterator(original);
+    while ((item = vector_iterator_next(&it)))
+        vector_append(clone, item);
+
+    // Return the cloned vector
+    return clone;
 }
 
 void
@@ -70,14 +94,13 @@ vector_clear(vector_t *vector)
 int
 vector_append(vector_t *vector, void *item)
 {
-
     // Sanity check
     if (!item)
         return vector->count;
 
     // Check if the vector has been initializated
     if (!vector->list) {
-        vector->list = malloc(sizeof(void *) * vector->limit);
+        vector->list = sng_malloc(sizeof(void *) * vector->limit);
     }
 
     // Check if we need to increase vector size
@@ -96,7 +119,7 @@ vector_append(vector_t *vector, void *item)
         vector->sorter(vector, item);
     }
 
-    return vector->count;
+    return vector->count - 1;
 }
 
 int
@@ -152,7 +175,7 @@ vector_set_sorter(vector_t *vector, void (*sorter) (vector_t *vector, void *item
 void
 vector_generic_destroyer(void *item)
 {
-    free(item);
+    sng_free(item);
 }
 
 void *
@@ -177,6 +200,12 @@ vector_first(vector_t *vector)
     return vector_item(vector, 0);
 }
 
+void *
+vector_last(vector_t *vector)
+{
+    return vector_item(vector, vector_count(vector) - 1);
+}
+
 int
 vector_index(vector_t *vector, void *item)
 {
@@ -274,7 +303,7 @@ vector_iterator_prev(vector_iter_t *it)
 
 void
 vector_iterator_set_filter(vector_iter_t *it, int
-(*filter)(void *item))
+                           (*filter)(void *item))
 {
     it->filter = filter;
 }
