@@ -42,8 +42,7 @@ typedef struct vector_iter vector_iter_t;
 /**
  * @brief Structure to hold a list of pointers
  */
-struct vector
-{
+struct vector {
     //! Number of elements in list
     u_int count;
     //! Total space in list (available + elements)
@@ -58,8 +57,7 @@ struct vector
     void (*sorter) (vector_t *vector, void *item);
 };
 
-struct vector_iter
-{
+struct vector_iter {
     //! Last requested position
     int current;
     //! Vector that's being iterated
@@ -83,6 +81,19 @@ vector_create(int limit, int step);
 void
 vector_destroy(vector_t *vector);
 
+/**
+ * @brief Clone a vector container
+ *
+ * The cloned vector will have its own storage of pointers
+ * **but the items in its list will be shared with the
+ * original vector**
+ *
+ * Use with caution, specially with vector that have sorter
+ * or destroyer functions.
+ */
+vector_t *
+vector_clone(vector_t *original);
+
 /**
  * @brief Remove all items of vector
  *
@@ -173,6 +184,12 @@ vector_set_item(vector_t *vector, int index, void *item);
 void *
 vector_first(vector_t *vector);
 
+/**
+ * @brief Return last item of the vector
+ */
+void *
+vector_last(vector_t *vector);
+
 /**
  * @brief Get the index of an item
  *
